<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

    /**
     * Array of plugin tabs on the options page.
     *
     * @since    1.0
     * @access   public
     * @var      array    $plugin_tabs    Array of plugin tabs.
     */
	public $plugin_tabs;

    /**
     * Slug for the plugin options page.
     *
     * @since    1.0
     * @access   public
     * @var      string    $options_slug    Slug for the plugin options page.
     */
    public $options_slug;

    /**
     * Link to the plugin options page.
     *
     * @since    1.0
     * @access   public
     * @var      string    $options_url    Link to the plugin options page.
     */
    public $options_url;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->options_slug = 'north-checkout';
        $this->options_url = 'admin.php?page=' . $this->options_slug;
        $this->plugin_tabs = [
            [
                'slug'  => 'north-checkout',
                'title' => _x( 'Overview', 'General statistic', 'north-checkout' ),
            ],
            [
                'slug'  => 'north-checkout-orders',
                'title' => _x( 'Orders', 'List of all orders', 'north-checkout' ),
            ],
            [
                'slug'  => 'north-checkout-products',
                'title' => _x( 'Products', 'List of all products', 'north-checkout' ),
            ],
            [
                'slug'  => 'north-checkout-coupons',
                'title' => _x( 'Coupons', 'List of all coupons', 'north-checkout' ),
            ],
            [
                'slug'  => 'north-checkout-settings',
                'title' => _x( 'Settings', 'Plugin general setting', 'north-checkout' ),
            ],
            [
                'slug'  => 'north-checkout-integration',
                'title' => _x( 'Integration', 'Integration manager', 'north-checkout' ),
            ],
        ];

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/north-checkout-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name . '-repeater', plugin_dir_url( __FILE__ ) . 'js/jquery.repeater.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name . '-lottie', plugin_dir_url( __FILE__ ) . 'js/lottie-player.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/north-checkout-admin.js', array( 'jquery' ), $this->version, false );
        wp_localize_script( $this->plugin_name, 'ajax_object',
            array(
                'ajaxurl' => admin_url( 'admin-ajax.php' ),
                'order_paged' => 2,
                'product_paged' => 2,
                'coupon_paged' => 2
            )
        );
        wp_enqueue_media();

	}

    /**
     * Create plugin tabs in admin area.
     *
     * @since    1.0
     */
    public function plugin_tabs() {
        $i = 0;
        $current_tab = 0;
        $columns = array_column($this->plugin_tabs, 'slug');

        if(isset($_GET['page'])) {
            $current_tab = array_search($_GET['page'], $columns);
        }

        foreach ( $this->plugin_tabs as $key => $tab ) {

            $active = $current_tab === $i ? ' north-tab-active' : '';

            printf(  '<a href="/wp-admin/admin.php?page=%s" class="north-tab %s">%s</a>', esc_html($tab['slug']), esc_html($active), esc_html($tab['title']) );
            $i++;
        }
    }

    /**
     * Function for including templates.
     *
     * @since    1.0
     *
     * @param $template string  name of template without .php
     * @param $dir string  name of template folder
     * @param $args array  arguments for template
     * @param $template_path string  path to plugin folder (by default is plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/)
     */
    public function template( $template, $dir = '', $args = array(), $template_path = '' ) {
        // @TODO: Refactor to remove extract().
        extract( $args, EXTR_OVERWRITE ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract
        $dir       = ! empty( $dir ) ? trailingslashit( $dir ) : $dir;
        $base_path = ! empty( $template_path ) ? $template_path : plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/';

        $path = $base_path . $dir . $template . '.php';
        include $path;
    }

    public function overview_page() {
        $this->template( 'overview', 'tabs');
    }

    public function orders_page() {
        $this->template( 'orders', 'tabs');
    }

    public function products_page() {
        $this->template( 'products', 'tabs');
    }

    public function coupons_page() {
        $this->template( 'coupons', 'tabs');
    }

    public function settings_page() {
        $this->template( 'settings', 'tabs');
    }

    public function integration_page() {
        $this->template( 'integration', 'tabs');
    }

    /**
     * Options page init.
     *
     */
    function admin_menu() {
        add_menu_page(__( 'North Checkout', 'north-checkout' ), __( 'North Checkout', 'north-checkout' ), 'edit_posts', $this->options_slug, array($this, 'overview_page'), plugin_dir_url(__FILE__) . 'images/mini-icon.png');
        add_submenu_page( $this->options_slug, __( 'Overview', 'north-checkout' ), __( 'Overview', 'north-checkout' ), 'edit_posts', $this->options_slug, array($this, 'overview_page') );
        add_submenu_page( $this->options_slug, __( 'Orders', 'north-checkout' ), __( 'Orders', 'north-checkout' ), 'edit_posts', $this->options_slug . '-orders', array($this, 'orders_page') );
        add_submenu_page( $this->options_slug, __( 'Products', 'north-checkout' ), __( 'Products', 'north-checkout' ), 'edit_posts', $this->options_slug . '-products', array($this, 'products_page') );
        add_submenu_page( $this->options_slug, __( 'Coupons', 'north-checkout' ), __( 'Coupons', 'north-checkout' ), 'edit_posts', $this->options_slug . '-coupons', array($this, 'coupons_page') );
        add_submenu_page( $this->options_slug, __( 'Settings', 'north-checkout' ), __( 'Settings', 'north-checkout' ), 'edit_posts', $this->options_slug . '-settings', array($this, 'settings_page') );
        add_submenu_page( $this->options_slug, __( 'Integration', 'north-checkout' ), __( 'Integration', 'north-checkout' ), 'edit_posts', $this->options_slug . '-integration', array($this, 'integration_page') );
    }

    public static function is_plugin_activated() {

        if(isset($_SERVER['SERVER_NAME']) && $_SERVER['SERVER_NAME'] === 'northplugins.com') {
            return true;
        }

        if(!get_option('north_licence_key')) {
            return false;
        }

        $api_params = array(
            'slm_action' => 'slm_check',
            'secret_key' => NORTH_SECRET_KEY,
            'license_key' => get_option('north_licence_key'),
            'registered_domain' => sanitize_text_field( wp_unslash( $_SERVER['SERVER_NAME'] ) ),
        );

        $response = wp_remote_get(add_query_arg($api_params, NORTH_LICENSE_SERVER_URL), array('timeout' => 20, 'sslverify' => false));

        if (is_wp_error($response)){
            return false;
        }

        $license_data = json_decode(wp_remote_retrieve_body($response));

        if($license_data->result == 'error') {
            return false;
        }else {
            foreach($license_data->registered_domains as $domain_data) {
                if($_SERVER['SERVER_NAME'] == $domain_data->registered_domain) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Activate plugin licence.
     *
     * @param $licence_key
     * @return mixed
     */
    function activate_product($licence_key) {

        $api_params = array(
            'slm_action' => 'slm_activate',
            'secret_key' => NORTH_SECRET_KEY,
            'license_key' => $licence_key,
            'item_reference' => 'North Plugin',
            'registered_domain' =>  isset($_SERVER['SERVER_NAME']) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_NAME'] ) ) : '',
        );

        $response = wp_remote_get(add_query_arg($api_params, NORTH_LICENSE_SERVER_URL), array('timeout' => 20, 'sslverify' => false));

        // Check for error in the response
        if (is_wp_error($response)){
            echo '<div class="north-error" style="display: block;">Licence key are not saved! Bad request</div>';
        }

        $license_data = json_decode(wp_remote_retrieve_body($response));

        if($license_data->result == 'error') {
            if($license_data->error_code == 40) {
                update_option('north_licence_key', $licence_key);
                echo '<div class="north-success" style="display: block;">Plugin was activated!</div>';
            }else {
                echo '<div class="north-error" style="display: block;">'.esc_html($license_data->message).'</div>';
            }
        }else {
            update_option('north_licence_key', $licence_key);
            echo '<div class="north-success" style="display: block;">Plugin was activated!</div>';
        }

        return $licence_key;
    }

    /**
     * Dectivate plugin licence.
     *
     */
    function deactivate_product() {

        $api_params = array(
            'slm_action' => 'slm_deactivate',
            'secret_key' => NORTH_SECRET_KEY,
            'license_key' => get_option('north_licence_key'),
            'registered_domain' => isset($_SERVER['SERVER_NAME']) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_NAME'] ) ) : '',
        );

        $response = wp_remote_get(add_query_arg($api_params, NORTH_LICENSE_SERVER_URL), array('timeout' => 20, 'sslverify' => false));

        // Check for error in the response
        if (is_wp_error($response)) {
            echo '<div class="north-error" style="display: block;">Licence are not deactivated! Bad request</div>';
        }

        $license_data = json_decode(wp_remote_retrieve_body($response));

        if ($license_data->result == 'error') {
            echo esc_html('<div class="north-error" style="display: block;">' . $license_data->message . '</div>');
        } else {
            delete_option('north_licence_key');
            echo '<div class="north-success" style="display: block;">Plugin was dectivated!</div>';
        }
    }

    /**
     * Convert currency code to symbol.
     *
     * @param string $cur_code
     * @return string currency symbol
     */
    public static function convert_code($cur_code) {

        if(strtolower($cur_code) === 'usd')
            return '$';
        elseif(strtolower($cur_code) === 'eur')
            return '€';
        elseif(strtolower($cur_code) === 'cad')
            return '$';
        elseif(strtolower($cur_code) === 'aud')
            return '$';
        elseif(strtolower($cur_code) === 'gbp')
            return '£';
        elseif(strtolower($cur_code) === 'dkk')
            return 'kr.';
        else 'ERR';

    }

    /**
     * Format currency code to symbol.
     *
     * @param $price
     * @param string $cur_code
     * @return string currency symbol
     */
    public static function formatted_price($price, $with_wrapper = true, $currency = null) {

        $cur_code = self::get_current_currency();
        $price = floatval($price);

        if($with_wrapper) {
            $price = '<span class="value">' . number_format($price, 2) . '</span>';
        }else {
            $price = number_format($price, 2);
        }

        if($currency != null) {
            if(strtolower($currency) === 'kr.') {
                return $price . ' kr.';
            } else {
                return $currency . $price;
            }
        }else {
            if(strtolower($cur_code) === 'usd')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'eur')
                return '€' . $price;
            elseif(strtolower($cur_code) === 'cad')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'aud')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'gbp')
                return '£' . $price;
            elseif(strtolower($cur_code) === 'dkk')
                return $price .' kr.';
            else 'ERR';
        }

    }

    /**
     * Format currency code to symbol.
     *
     * @param $price
     * @param string $cur_code
     * @return string currency symbol
     */
    public static function formatted_coupon_price($price, $with_wrapper = true, $currency = null) {

        $cur_code = self::get_current_currency();
        $price = floatval($price);

        if($with_wrapper) {
            $price = '<span class="discount-value">' . number_format($price, 2) . '</span>';
        }else {
            $price = number_format($price, 2);
        }

        if($currency != null) {
            if(strtolower($currency) === 'kr.') {
                return $price . ' kr.';
            } else {
                return $currency . $price;
            }
        }else {
            if(strtolower($cur_code) === 'usd')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'eur')
                return '€' . $price;
            elseif(strtolower($cur_code) === 'cad')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'aud')
                return '$' . $price;
            elseif(strtolower($cur_code) === 'gbp')
                return '£' . $price;
            elseif(strtolower($cur_code) === 'dkk')
                return $price .' kr.';
            else 'ERR';
        }

    }

    /**
     * Get current currency.
     *
     * @return string currency code
     */
    public static function get_current_currency() {

        return !empty(get_option('current_currency')) ? get_option('current_currency') : 'USD';

    }

    /**
     * Set current currency.
     *
     * @param $currency
     */
    public static function set_current_currency($currency) {

        update_option('current_currency', $currency);

    }

    /**
     * Store plugin google api key.
     *
     * @param $north_google_key
     * @return mixed
     */
    function north_google_key_save($north_google_key) {
        update_option('north_google_key', $north_google_key);

        return $north_google_key;
    }

    /**
     * Store plugin google api key.
     *
     * @param $test_mode
     * @return mixed
     */
    function north_change_active_keys($test_mode) {

        if(!empty($test_mode)) {
            update_option('stripe_public_active', get_option('stripe_test_public_key'));
            update_option('stripe_private_active', get_option('stripe_test_private_key'));
        }else {
            update_option('stripe_public_active', get_option('stripe_public_key'));
            update_option('stripe_private_active', get_option('stripe_private_key'));
        }

    }


    /**
     * Store plugin google api key.
     *
     * @param $res
     * @param $action
     * @param $args
     * @return mixed
     */
    function north_plugin_info( $res, $action, $args ){
        // do nothing if this is not about getting plugin information
        if( 'plugin_information' !== $action ) {
            return false;
        }

        $plugin_slug = 'north-checkout'; // we are going to use it in many places in this function

        // do nothing if it is not our plugin
        if( $plugin_slug !== $args->slug ) {
            return false;
        }

        // trying to get from cache first
        if( false == $remote = get_transient( 'north_update_' . $plugin_slug ) ) {

            // info.json is the file with the actual plugin information on your server
            $remote = wp_remote_get( 'https://northplugins.com/app/info.json', array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    ) )
            );

            if ( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) ) {
                set_transient( 'north_update_' . $plugin_slug, $remote, 43200 ); // 12 hours cache
            }

        }

        if( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) ) {

            $remote = json_decode( $remote['body'] );
            $res = new stdClass();

            $res->name = $remote->name;
            $res->slug = $plugin_slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = '<a href="https://northplugins.com/">North Plugins</a>';
            $res->author_profile = 'https://profiles.wordpress.org/zellocoding';
            $res->download_link = $remote->download_url;
            $res->trunk = $remote->download_url;
            $res->requires_php = '5.3';
            $res->last_updated = $remote->last_updated;
            $res->sections = array(
                'description' => $remote->sections->description,
                'installation' => $remote->sections->installation,
                'changelog' => $remote->sections->changelog,
                'screenshots' => $remote->sections->screenshots,
            );
            $res->contributors = array(
                array(
                    'display_name' => 'zellocoding',
                    'profile' => 'https://profiles.wordpress.org/zellocoding',
                    'avatar' => 'https://secure.gravatar.com/avatar/0ae87009b37f2dc5d4d9b7722d65689c?s=100&d=mm&r=g',
                ),
                array(
                    'display_name' => 'kelleymuro',
                    'profile' => 'https://profiles.wordpress.org/kelleymuro/',
                    'avatar' => 'https://secure.gravatar.com/avatar/ab86267f60175c6b706b7fbe8e874183?s=100&d=mm&r=g',
                )
            );

            $res->banners = array(
                'low' => 'https://northplugins.com/app/banner-772x250.jpg',
                'high' => 'https://northplugins.com/app/banner-1544x500.jpg'
            );

            return $res;
        }

        return false;
    }

    /**
     * Store plugin google api key.
     *
     * @param $transient
     * @return mixed
     */
    function north_push_update( $transient ){

        if ( empty($transient->checked ) ) {
            return $transient;
        }

        // trying to get from cache first, to disable cache comment 10,20,21,22,24
        if( false == $remote = get_transient( 'north_upgrade_north-checkout' ) ) {

            // info.json is the file with the actual plugin information on your server
            $remote = wp_remote_get( 'https://northplugins.com/app/info.json', array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    ) )
            );

            if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && !empty( $remote['body'] ) ) {
                set_transient( 'north_upgrade_north-checkout', $remote, 43200 ); // 12 hours cache
            }

        }

        if( !isset($remote->errors) ) {

            $remote = json_decode( $remote['body'] );

            // your installed plugin version should be on the line below! You can obtain it dynamically of course
            if( $remote && version_compare( NORTH_CHECKOUT_VERSION, $remote->version, '<' ) && version_compare($remote->requires, get_bloginfo('version'), '<' ) ) {
                $res = new stdClass();
                $res->slug = 'north-checkout';
                $res->plugin = 'north-checkout/north-checkout.php'; // it could be just YOUR_PLUGIN_SLUG.php if your plugin doesn't have its own directory
                $res->new_version = $remote->version;
                $res->tested = $remote->tested;
                $res->package = $remote->download_url;
                $transient->response[$res->plugin] = $res;
                //$transient->checked[$res->plugin] = $remote->version;
            }

        }
        return $transient;
    }

    /**
     * Store plugin google api key.
     *
     * @param $upgrader_object
     * @param $options
     * @return mixed
     */
    function north_after_update( $upgrader_object, $options ) {
        if ( $options['action'] == 'update' && $options['type'] === 'plugin' )  {
            // just clean the cache when new plugin version is installed
            delete_transient( 'north_upgrade_north-checkout' );
        }
    }

    function switch_payment_mode() {

        $payment_system = $_POST['payment_system']; // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $value = isset($_POST['payment_value']) ? sanitize_text_field( $_POST['payment_value'] ) : '';  // phpcs:ignore WordPress.Security.NonceVerification.Missing

        if($payment_system == 'stripe') {
            $this->north_change_active_keys($value);
        }

        update_option('test_' . $payment_system, $value);

        wp_die();
    }

}
