<?php
/**
 * Coupon class.
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Coupon {

    /**
     * Coupon id.
     *
     * @access   private
     * @var      integer    $coupon_id    coupon id.
     */
    private $coupon_id;

    /**
     * Query coupons object.
     *
     * @access   private
     * @var      object    $coupon_query    Query coupons array.
     */
    private $coupon_query;

    function __construct($coupon_id = null) {
        $this->coupon_id = $coupon_id;
    }

    /**
     * Register Coupon Custom Post Type with settings.
     *
     */
    function register_coupon() {

        register_post_type( 'north_coupon', [
            'label'  => null,
            'labels' => [
                'name'               => 'Coupon',
                'singular_name'      => 'Coupon',
            ],
            'public'              => true,
            'show_in_menu'        => false,
            'show_in_rest'        => true,
            'hierarchical'        => false,
            'supports'            => [ 'title', 'editor', 'custom-fields'],
            'rewrite'             => true,
            'query_var'           => true,
        ] );

        register_post_status( 'active', array(
            'label'                     => _x( 'Active', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_coupon'),
            'label_count'               => _n_noop( 'Active <span class="count">(%s)</span>', 'Active <span class="count">(%s)</span>' ),
        ) );

        register_post_status( 'not_active', array(
            'label'                     => _x( 'Not Active', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_coupon'),
            'label_count'               => _n_noop( 'Not active <span class="count">(%s)</span>', 'Not active <span class="count">(%s)</span>' ),
        ) );
    }

    /**
     * Prepare order to transfer to Zapier.
     */
    function coupon_prepare_zap($coupon) {

        $coupon_obj = new North_Checkout_Coupon($coupon->ID);
        $coupon_type = get_post_meta($coupon->ID, 'coupon_type', true);

        $coupon_data = array(
            'id' => $coupon->ID,
            'title' => $coupon->post_title,
            'status' => $coupon->post_status,
            'date_created' => $coupon->post_date,
            'date_modified' => $coupon->post_modified,
            'coupon_type' => $coupon_type,
            'coupon_price' => strval($coupon_obj->get_price()),
            'coupon_currency' => strval(North_Checkout_Admin::get_current_currency()),
            'coupon_currency_symb' => North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency()),
            'coupon_quantity' => strval($coupon_obj->get_quantity()),
        );

        return apply_filters('north_prepare_zapier_coupon', $coupon_data);

    }

    /**
     * Register route for Coupon.
     */
    function register_coupon_route() {

        register_rest_route( 'north-checkout/v1', '/coupons/', array(
            'methods'  => 'GET',
            'callback' => array($this, 'get_rest_coupons'),
            'permission_callback' => '__return_true'
        ) );
    }

    /**
     * Register route for Coupon.
     */
    function get_rest_coupons( WP_REST_Request $request ) {

        $coupons = get_posts( array(
            'numberposts' => -1,
            'post_type'   => 'north_coupon',
            'post_status' => 'any',
            'suppress_filters' => true,
        ) );

        $coupon_data = array();

        foreach ($coupons as $coupon) {
            $coupon_data[] = $this->coupon_prepare_zap($coupon);
        }

        $response = rest_ensure_response( $coupon_data );
        $response->set_status( 200 );
        return $response;
    }

    /**
     * Function for including templates.
     *
     * @since    1.0
     *
     * @param $template string  name of template without .php
     * @param $dir string  name of template folder
     * @param $args array  arguments for template
     * @param $template_path string  path to plugin folder (by default is plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/)
     */
    public function template( $template, $dir = '', $args = array(), $template_path = '' ) {
        // @TODO: Refactor to remove extract().
        extract( $args, EXTR_OVERWRITE ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract
        $dir       = ! empty( $dir ) ? trailingslashit( $dir ) : $dir;
        $base_path = ! empty( $template_path ) ? $template_path : plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/';

        $path = $base_path . $dir . $template . '.php';
        include $path;
    }

    /**
     * Get single coupon by id.
     *
     * @since   1.0
     *
     * @param   $coupon_id integer Coupon Id
     *
     * @return array
     */
    function get_coupon($coupon_id) {

        return get_post_meta($coupon_id);

    }

    /**
     * Send hook to zapier.
     *
     * @since   1.0
     *
     * @param   $post_id integer Coupon Id
     *
     */
    function send_hook($post_id) {

        $hooks = get_option('north_hooks');

        if ($hooks) {
            $coupon = get_post($post_id);
            $coupon_data = $this->coupon_prepare_zap($coupon);

            foreach ($hooks as $hook) {

                if (isset($hook["hook_type"]) && $hook["hook_type"] === 'coupon_created' && isset($hook["hook"])) {
                    wp_remote_post($hook["hook"], array(
                            'method' => 'POST',
                            'timeout' => 45,
                            'redirection' => 5,
                            'headers' => array(),
                            'body' => $coupon_data,
                        )
                    );
                }
            }
        }
    }

    /**
     * Create new coupon.
     *
     * @param $coupon_name
     * @param string $status Coupon status slug ( active | not_active)
     * @param array $data Array of coupon meta
     * @return int|WP_Error Coupon id or WP_Error
     */
    function create_coupon($coupon_name, $status = 'active', $data) {

        do_action('north_before_coupon_created', $coupon_name, $data, $status);

        $coupon_data = array(
            'post_type'     => 'north_coupon',
            'post_title'    => apply_filters('north_create_coupon_name', wp_strip_all_tags( $coupon_name )),
            'post_status'   => $status
        );

        $coupon_id = wp_insert_post( $coupon_data );

        $data = apply_filters('north_create_coupon_data', $data, $coupon_id);

        if ( is_wp_error($coupon_id) ) {
            do_action('north_coupon_created_failed', $coupon_id, $data);
        }else {
            do_action('north_coupon_created_success', $coupon_id, $data);

            foreach($data as $key => $value) {
                update_post_meta($coupon_id, $key, $value);
            }
        }

        return $coupon_id;
    }

    /**
     * Update coupon.
     *
     * @param $coupon_name
     * @param string $status Coupon status slug ( active | not_active)
     * @param array $data Array of coupon meta
     * @return int|WP_Error Coupon id or WP_Error
     */
    function update_coupon($coupon_id = null, $coupon_name, $status = 'active', $data) {

        do_action('north_before_coupon_updated', $coupon_id, $coupon_name, $data, $status);

        $coupon_id = !empty($coupon_id) ? $coupon_id : $this->coupon_id;

        $coupon_data = array(
            'ID'            => $coupon_id,
            'post_title'    => wp_strip_all_tags( $coupon_name ),
            'post_status'   => $status
        );

        $data = apply_filters('north_update_coupon_data', $data, $coupon_id);

        wp_update_post( $coupon_data );

        foreach($data as $key => $value) {
            update_post_meta($coupon_id, $key, $value);
        }

        do_action('north_after_coupon_updated', $coupon_id, $coupon_name, $data, $status);

        return $coupon_id;
    }

    /**
     * Delete coupon.
     *
     * @param integer $coupon_id Coupon id
     * @return false|WP_Post|null
     */
    function delete_coupon($coupon_id) {

        $coupon_id = isset($_POST['delete_id']) ? sanitize_text_field($_POST['delete_id']) : $coupon_id; // phpcs:ignore WordPress.Security.NonceVerification.Missing

        do_action('north_before_coupon_deleted', $coupon_id);

        if (isset($_POST['delete_id'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_delete_post($coupon_id);
            wp_die();
        }

        $result = wp_delete_post($coupon_id);

        do_action('north_after_coupon_deleted', $coupon_id);

        return $result;
    }

    /**
     * Get coupon loop.
     *
     * @param int $per_page
     * @param bool $is_array
     * @param array $meta
     * @return array
     */
    function get_coupons( $per_page = 8, $is_array = false, $meta = null, $compare = null) {

        $paged = isset($_POST['paged']) ? sanitize_text_field($_POST['paged']) : 1;  // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $posts_per_page = isset($_POST['posts_per_page']) ? sanitize_text_field($_POST['posts_per_page']) : $per_page; // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $meta_query = array();

        if($meta) {
            foreach ($meta as $key => $value) {
                $meta_query[] = array(
                    'key' => $key,
                    'value' => $value,
                    'compare' => $compare,
                );
            }
        }

        $args = array(
            'post_type'         => 'north_coupon',
            'posts_per_page'    => $posts_per_page,
            'paged'             => $paged,
            'meta_query'        => $meta_query
        );

        $args = apply_filters('north_get_coupons', $args);

        $this->coupon_query = new WP_Query( $args );

        if($this->coupon_query->have_posts()) {

            if($is_array) {

                $result = array();

                while($this->coupon_query->have_posts()) : $this->coupon_query->the_post();

                    $result[get_the_ID()] = esc_html__( get_the_title(), 'north-checkout' );

                endwhile;

                wp_reset_query();

                $result['0'] = esc_html__('Select Coupon', 'north-checkout' );

                return $result;

            }else {

                while($this->coupon_query->have_posts()) : $this->coupon_query->the_post();

                    $coupon_id = get_the_ID();

                    $args = array(
                        'coupon_price' => $this->get_price_html($coupon_id),
                        'coupon_type' => $this->get_type(),
                    );

                    $this->template('single-coupon','coupon', $args);

                endwhile;

                wp_reset_query();

            }

        }else {
            if( isset($_POST['paged'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
                echo 'error';
            }elseif( $is_array ){
                return;
            }else {
                return 0;
            }
        }

        if( isset($_POST['posts_per_page']) ) // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_die();

    }

    /**
     * Get order loop pagination.
     */
    function get_pagination() {
        if ($this->coupon_query->max_num_pages > 1) {
            $this->template('coupon-pagination','coupon');
        }
    }

    /**
     * Get coupon id.
     *
     * @param null $coupon_id
     *
     * @return false|int|null
     */
    function get_id($coupon_id = null) {
        if (isset($coupon_id) && $coupon_id != null) {
            return $coupon_id;
        }elseif (isset($this->coupon_id)){
            return $this->coupon_id;
        }else {
            return get_the_ID();
        }
    }

    /**
     * Get coupon price.
     *
     * @param $coupon_id
     * @param bool $without_currency
     * @param string $currency_position
     *
     * @return string coupon price with currency
     */
    function get_price($coupon_id = null) {

        $id = $this->get_id($coupon_id);
        $price = floatval(get_post_meta($id, 'coupon_price', true));

        return apply_filters('north_get_coupon_price', $price, $coupon_id);
    }

    /**
     * Get coupon price.
     *
     * @param $coupon_id
     * @param bool $without_currency
     * @param string $currency_position
     *
     * @return string coupon price with currency
     */
    function get_price_html($coupon_id = null) {

        $id = $this->get_id($coupon_id) ;
        $price = '';

        if($this->get_type($id) == 'fixed') {
            $price = North_Checkout_Admin::formatted_coupon_price(get_post_meta($id, 'coupon_price', true));
        }else {
            $price = get_post_meta($id, 'coupon_price', true) . '%';
        }

        return apply_filters('north_get_coupon_price_html', $price, $coupon_id);
    }

    /**
     * Get coupon quantity.
     *
     * @param $coupon_id
     * @return string coupon quantity
     */
    function get_limit($coupon_id = null) {

        $id = $this->get_id($coupon_id);

        return get_post_meta($id, 'coupon_limit', true);
    }

    /**
     * Get coupon type.
     *
     * @param integer $coupon_id
     *
     * @return string coupon type
     */
    function get_type($coupon_id = null) {

        $id = $this->get_id($coupon_id);

        return get_post_meta($id, 'coupon_type', true);
    }

}
