<?php

/**
 * Order class.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Order {

    /**
     * Query orders object.
     *
     * @access   private
     * @var      object    $order_query    Query orders array.
     */
    private $order_query;

    /**
     * Register Order Custom Post Type with settings.
     *
     * @since    1.0
     */

    function register_order() {

        register_post_type( 'north_order', [
            'label'  => null,
            'labels' => [
                'name'               => 'Order',
                'singular_name'      => 'Order',
            ],
            'public'              => true,
            'show_in_menu'        => false,
            'show_in_rest'        => false,
            'hierarchical'        => false,
            'supports'            => [ 'title', 'editor', 'custom-fields'],
            'query_var'           => true,
        ] );

        register_post_status( 'complete', array(
            'label'                     => _x( 'Complete', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_order'),
            'label_count'               => _n_noop( 'Order Complete <span class="count">(%s)</span>', 'Order Complete <span class="count">(%s)</span>' ),
        ) );

        register_post_status( 'failed', array(
            'label'                     => _x( 'Failed', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_order'),
            'label_count'               => _n_noop( 'Order Failed <span class="count">(%s)</span>', 'Order Failed <span class="count">(%s)</span>' ),
        ) );

        register_post_status( 'warning', array(
            'label'                     => _x( 'Warning', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_order'),
            'label_count'               => _n_noop( 'Order Pending <span class="count">(%s)</span>', 'Order Pending <span class="count">(%s)</span>' ),
        ) );
    }

    /**
     * Prepare order to transfer to Zapier.
     */
    function order_prepare_zap($order) {
        if ($customer_address = get_post_meta($order->ID, 'customer_address', true)) {
            $customer_address = json_decode($customer_address);
        }

        $product_id = get_post_meta($order->ID, 'product_id', true);
        $bump_id = get_post_meta($order->ID, 'bump_id', true);
        $product = null;
        $bump = null;

        if (!empty($product_id)) {
            $product = new North_Checkout_Product($product_id);
        }

        if (!empty($bump_id)) {
            $bump = new North_Checkout_Product($bump_id);
        }

        $order_data = array(
            'id' => $order->ID,
            'title' => $order->post_title,
            'slug' => $order->post_name,
            'status' => $order->post_status,
            'date_created' => $order->post_date,
            'date_modified' => $order->post_modified,
            'customer_first_name' => get_post_meta($order->ID, 'customer_first', true),
            'customer_last_name' => get_post_meta($order->ID, 'customer_last', true),
            'customer_full_name' => get_post_meta($order->ID, 'customer_name', true),
            'customer_email' => get_post_meta($order->ID, 'customer_email', true),
            'customer_phone' => get_post_meta($order->ID, 'customer_phone', true),
            'customer_address' => isset($customer_address->full) ? $customer_address->full : null,
            'customer_address_line_2' => isset($customer_address->line_2) ? $customer_address->line_2 : null,
            'customer_country' => isset($customer_address->country) ? $customer_address->country : null,
            'customer_city' => isset($customer_address->city) ? $customer_address->city : null,
            'customer_state' => isset($customer_address->state) ? $customer_address->state : null,
            'customer_zip' => isset($customer_address->zip) ? $customer_address->zip : null,
            'product_id' => $product_id,
            'product_name' => get_post_meta($order->ID, 'product_name', true),
            'product_price' => get_post_meta($order->ID, 'product_subtotal', true),
            'product_currency' => get_post_meta($order->ID, 'product_currency', true),
            'product_quantity' => get_post_meta($order->ID, 'product_quantity', true) ? get_post_meta($order->ID, 'product_quantity', true) : '1',
            'product_type' => get_post_meta($order->ID, 'product_type', true),
            'bump_purchased' => isset($bump_id) && !empty($bump_id),
            'bump_id' => $bump_id,
            'bump_name' => get_post_meta($order->ID, 'bump_name', true),
            'bump_price' => get_post_meta($order->ID, 'bump_subtotal', true),
            'bump_currency' => get_post_meta($order->ID, 'bump_currency', true),
            'bump_type' => get_post_meta($order->ID, 'bump_type', true),
            'custom_fields' => get_post_meta($order->ID, 'flex_info', true),
            'product_payment_status' => get_post_meta($order->ID, 'product_payment_status', true),
            'bump_payment_status' => get_post_meta($order->ID, 'bump_payment_status', true),
        );

//        if ($customer_info = get_post_meta($order->ID, 'customer_info', true)) {
//            $customer_info = json_decode($customer_info);
//
//            foreach ($customer_info as $key => $value) {
//
//                if($key == in_array($key, array('checkout_first', 'checkout_last', 'checkout_email'))) {
//                    continue;
//                }
//
//                if(is_array($value)) {
//                    $order_data[$key] = implode(', ', $value);
//                }else {
//                    $order_data[$key] = $value;
//                }
//            }
//        }

        $variations = get_post_meta($order->ID, 'variations', true );
        $variation_string = '';

        if( $variations && $variations !== '[]') {

            $variations = json_decode($variations);

            if(is_object($variations)) {
                $variation_string = '( ';
                foreach($variations as $key => $value) {
                    $variation_string .= ucfirst($key) . ': ' . $value . '; ';
                }

                $variation_string .= ')';
            }

            $order_data['product_variation'] = $variation_string;

        }

        return apply_filters('north_prepare_zapier_order', $order_data);
    }

    /**
     * Register route for Order.
     */
    function register_order_route() {

        register_rest_route( 'north-checkout/v1', '/orders/', array(
            'methods'  => 'GET',
            'callback' => array($this, 'get_rest_orders'),
            'permission_callback' => '__return_true'
        ) );
    }

    /**
     * Register route for Order.
     */
    function get_rest_orders( WP_REST_Request $request ) {

        $orders = get_posts( array(
            'numberposts' => -1,
            'post_type'   => 'north_order',
            'post_status' => 'any',
            'suppress_filters' => true,
        ) );

        $order_data = array();

        foreach ($orders as $order) {
            $order_data[] = $this->order_prepare_zap($order);
        }

        $response = rest_ensure_response( $order_data );
        $response->set_status( 200 );
        return $response;
    }

    /**
     * Function for including templates.
     *
     * @since    1.0
     *
     * @param $template string  name of template without .php
     * @param $dir string  name of template folder
     * @param $args array  arguments for template
     * @param $template_path string  path to plugin folder (by default is plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/)
     */
    public function template( $template, $dir = '', $args = array(), $template_path = '' ) {
        // @TODO: Refactor to remove extract().
        extract( $args, EXTR_OVERWRITE ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract
        $dir       = ! empty( $dir ) ? trailingslashit( $dir ) : $dir;
        $base_path = ! empty( $template_path ) ? $template_path : plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/';

        $path = $base_path . $dir . $template . '.php';
        include $path;
    }

    /**
     * Get single order by id.
     *
     * @since   1.0
     *
     * @param   $order_id integer Order Id
     *
     * @return array
     */
    function get_order($order_id) {
        $order_data = get_post_meta($order_id);

        return $order_data;
    }

    /**
     * Create new order.
     *
     * @param   $flex_info array Customer information ( name | email | shipping_address | billing_address | phone )
     * @param   $customer_address
     * @param   $order_data
     * @param   $status string Order status slug ( pending | complete | failed )
     * @return  int|WP_Error Order id or WP_Error
     * @since   1.0
     *
     */
    function create_order($order_data, $customer_address, $flex_info, $status, $payment_method = null) {

        do_action('north_before_order_created', $order_data, $customer_address, $flex_info, $status);

        $orders_count = wp_count_posts( 'north_order' );
        $order_number = apply_filters('north_order_number', intval($orders_count->pending) + intval($orders_count->complete) + intval($orders_count->failed) + 1);
        $method = isset($payment_method) ? $payment_method : (isset($order_data['payment_method']) ? 'PayPal' : 'Stripe');

        $meta = array(
            'card_last_four'            => $order_data['card_last_four'],
            'card_brand'                => $order_data['card_brand'],
            'flex_info'                 => json_encode($flex_info),
            'customer_name'             => $order_data['checkout_first'] . " " . $order_data['checkout_last'],
            'customer_first'            => $order_data['checkout_first'],
            'customer_last'             => $order_data['checkout_last'],
            'customer_phone'            => $order_data['checkout_phone'],
            'customer_address'          => json_encode($customer_address),
            'product_trans_id'          => $order_data['product_trans_id'],
            'product_price'             => $order_data['product_price'],
            'product_subtotal'          => $order_data['product_subtotal'],
            'product_quantity'          => $order_data['product_quantity'],
            'product_currency'          => $order_data['product_currency'],
            'product_type'              => $order_data['product_type'],
            'product_price_html'        => $order_data['product_price_html'],
            'product_payment_status'    => $order_data['product_payment_status'],
            'bump_payment_status'       => $order_data['bump_payment_status'],
            'payment_method'            => $method,
            'subtotal'                  => $order_data['subtotal'],
            'total'                     => $order_data['total'],
        );

        if( isset($order_data['product_name']) ) {
            $meta['product_id'] = $order_data['dynamic'];
            $meta['product_name'] = $order_data['product_name'];
        }else {
            $product_id = $order_data['product_id'];
            $meta['product_id'] = $product_id;
            $meta['product_name'] = get_the_title($product_id);
        }

        if( isset($order_data['checkout_email']) ) {
            $meta['customer_email'] = $order_data['checkout_email'];
        }

        if(isset($product_id)) {
            $this->increase_sale_counter($product_id);
            $this->update_product_quantity($product_id, $order_data['product_quantity']);
        }

        if($coupon = $order_data['coupon']) {
            $meta['coupon'] = $coupon;
            if(!empty($coupon)) {
                $coupon = get_page_by_title($coupon, OBJECT, 'north_coupon');
                $coupon_id = $coupon->ID;

                if(!empty($coupon_id)) {
                    $this->update_coupon_limit($coupon_id);
                }
            }
        }

        if( isset($order_data['bump_id']) ) {
            $meta['bump_id']            = $order_data['bump_id'];
            $meta['bump_name']          = get_the_title($order_data['bump_id']);
            $meta['bump_price_html']    = $order_data['bump_price_html'];
            $meta['bump_subtotal']      = $order_data['bump_subtotal'];
            $meta['bump_currency']      = $order_data['bump_currency'];
            $meta['bump_type']          = $order_data['bump_type'];
            $meta['bump_trans_id']      = $order_data['bump_trans_id'];

            $this->increase_sale_counter($order_data['bump_id']);
        }

        if( isset($order_data['variations']) ) {
            $meta['variations'] = json_encode($order_data['variations']);
        }

        $order_args = array(
            'post_type'     => 'north_order',
            'post_title'    => apply_filters('north_create_order_name', wp_strip_all_tags( '#' . $order_number )),
            'post_status'   => apply_filters('north_create_order_status',$status),
            'meta_input'    => apply_filters('north_create_order_meta', $meta)
        );

        $order_id = wp_insert_post( $order_args, true );
        $order = get_post($order_id);
        $hooks = get_option('north_hooks');

        if($hooks) {
            $order_data = $this->order_prepare_zap($order);

            foreach($hooks as $hook) {

                if(isset($hook["hook_type"]) && $hook["hook_type"] === 'order_created' && isset($hook["hook"])) {
                    wp_remote_post( $hook["hook"], array(
                            'method' => 'POST',
                            'timeout' => 45,
                            'redirection' => 5,
                            'headers' => array(),
                            'body' => $order_data,
                        )
                    );
                }
            }
        }

        do_action('north_after_order_created', $order_data, $customer_address, $flex_info, $status);

        return $order_id;
    }

    /**
     * Get order loop.
     *
     * @param int $per_page
     */
    function get_orders( $per_page = 8 ) {

        if (isset($_POST['filter'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            parse_str(  $_POST['filter'] , $filter); // phpcs:ignore WordPress.Security.NonceVerification.Missing, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized
        }

        $paged = isset($_POST['paged']) ? sanitize_text_field( $_POST['paged'] ) : 1; // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $posts_per_page = isset($_POST['posts_per_page']) ? sanitize_text_field( $_POST['posts_per_page'] ) : $per_page; // phpcs:ignore WordPress.Security.NonceVerification.Missing

        $args = array(
            'post_type' => 'north_order',
            'posts_per_page' => $posts_per_page,
            'paged' => $paged,
            'post_status' => 'any'
        );

        if (!empty($filter["filter_name"])) {
            $args['s'] = $filter["filter_name"];
        }

        if (!empty($filter["filter_customer"])) {
            $args['meta_query'] = array(
                'relation' => 'OR',
                array(
                    'key'     => 'customer_name',
                    'value'   => $filter["filter_customer"],
                    'compare' => 'LIKE',
                ),
            );
        }

        $args = apply_filters('north_get_orders', $args);

        $this->order_query = new WP_Query( $args );

        if($this->order_query->have_posts()) {

            while($this->order_query->have_posts()) : $this->order_query->the_post();
                $args = array(
                    'variations' => $this->get_variations_string(get_the_ID()),
                );

                $this->template('single-order','order', $args);
            endwhile;

            wp_reset_query();

        }else {
            if( isset($_POST['action']) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
                echo 'error';
            }else {
                echo '<div class="north-error" style="display: block;">Orders not found!</div>';
            }
        }

        if( isset($_POST['action']) ) // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_die();

    }

    /**
     * Get order loop pagination.
     */
    function get_pagination() {
        if ($this->order_query->max_num_pages > 1) {
            $this->template('order-pagination','order');
        }
    }

    /**
     * Get number of completed orders.
     */
    public static function get_completed_orders() {
        $args = array(
            'post_type' => 'north_order',
            'posts_per_page' => -1,
            'post_status' => 'complete'
        );

        $args = apply_filters('north_get_completed_orders', $args);

        $order_query = new WP_Query( $args );
        $posts_found = $order_query->found_posts;

        wp_reset_query();

        return $order_query->found_posts;
    }

    /**
     * Get total revenue.
     */
    public static function get_total_revenue($without_currency = false, $currency_position = 'right') {
        $args = array(
            'post_type' => 'north_order',
            'posts_per_page' => -1,
            'post_status' => 'complete'
        );

        $order_query = new WP_Query( $args );

        $total = 0;

        while($order_query->have_posts()) : $order_query->the_post();
            $total += floatval( get_post_meta( get_the_ID(), 'total', true ) );
        endwhile;


        wp_reset_query();

        $result = null;

        if($without_currency) {
            $result = $total;
        }else {
            if($currency_position === 'right') {
                $result = $total . get_post_meta( get_the_ID(), 'product_currency', true );
            }else {
                $result = get_post_meta( get_the_ID(), 'product_currency', true ) . $total;
            }
        }

        return apply_filters('north_get_total_revenue', $result, $total, $without_currency, $currency_position);
    }

    /**
     * Increase sale counter for product.
     */
    public function increase_sale_counter($product_id) {
        $sale_counter = intval(get_post_meta($product_id, 'sale_counter', true));

        if( empty($sale_counter) ) {
            $sale_counter = 1;
        }else {
            $sale_counter++;
        }

        update_post_meta($product_id, 'sale_counter', $sale_counter);

        do_action('north_increase_sale_counter', $product_id, $sale_counter);
    }

    /**
     * Update product quantity after created order.
     *
     * @param $product_id
     * @param $quantity
     */
    public function update_product_quantity($product_id, $quantity) {
        $current_quantity = get_post_meta($product_id, 'product_quantity', true);

        if(!empty($current_quantity)) {
            $updated_quantity = intval($current_quantity) - intval($quantity);

            if($updated_quantity >= 0) {
                update_post_meta($product_id, 'product_quantity', $updated_quantity);
            }
            else {
                update_post_meta($product_id, 'product_quantity', 0);
            }
        }
    }

    /**
     * Update product quantity after created order.
     *
     * @param $coupon_id
     * @param $limit
     */
    public function update_coupon_limit($coupon_id) {
        $current_limit = get_post_meta($coupon_id, 'coupon_limit', true);

        if(!empty($current_limit)) {
            $updated_limit = intval($current_limit) - 1;

            if($updated_limit >= 0) {
                update_post_meta($coupon_id, 'coupon_limit', $updated_limit);
            }
            else {
                update_post_meta($coupon_id, 'coupon_limit', 0);
            }
        }
    }

    public function get_variations_string($order_id) {
        $variation_string = '';

        if( get_post_meta($order_id, 'variations', true ) && get_post_meta($order_id, 'variations', true ) !== '[]') {

            $variations = json_decode(get_post_meta($order_id, 'variations', true ));

            if(is_object($variations)) {
                $variation_string = '<br><b>Variations:</b> ';

                foreach($variations as $key => $value) {
                    $variation_string .= ucfirst(str_replace("_"," ", $key) ) . ' - ' . $value . ' | ';
                }

                $variation_string = rtrim($variation_string, '| ');

            }

            return apply_filters('north_variation_html', $variation_string);

        }

        return false;
    }

    /**
     * Delete order.
     *
     * @param integer $order_id Order id
     * @return false|WP_Post|null
     */
    function delete_order($order_id) {

        $order_id = isset($_POST['delete_id']) ? sanitize_text_field($_POST['delete_id']) : $order_id; // phpcs:ignore WordPress.Security.NonceVerification.Missing

        do_action('north_before_order_deleted', $order_id);

        if (isset($_POST['delete_id'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_delete_post($order_id);
            wp_die();
        }

        $result = wp_delete_post($order_id);

        do_action('north_after_order_deleted', $order_id);

        return $result;

    }

}
