<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the Paypal
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Paypal {

    /**
     * Paypal auth.
     *
     * @since    1.0
     * @access   public
     * @var      string    $paypal_auth    Base64 Paypal Credentials.
     */
    private $paypal_auth;

    /**
     * Paypal access token.
     *
     * @since    1.0
     * @access   public
     * @var      string    $access_token    Paypal access token.
     */
    private $access_token_test;

    /**
     * Paypal access token.
     *
     * @since    1.0
     * @access   public
     * @var      string    $access_token    Paypal access token.
     */
    private $access_token_live;

    /**
     * Paypal test client_id.
     *
     * @since    1.0
     * @access   public
     * @var      string    $client_test    Paypal test client_id.
     */
    private $client_test;

    /**
     * Paypal test secret_id.
     *
     * @since    1.0
     * @access   public
     * @var      string    $secret_test    Paypal test secret_id.
     */
    private $secret_test;

    /**
     * Paypal live client_id.
     *
     * @since    1.0
     * @access   public
     * @var      string    $client_live    Paypal live client_id.
     */
    private $client_live;

    /**
     * Paypal live secret_id.
     *
     * @since    1.0
     * @access   public
     * @var      string    $secret_live    Paypal live secret_id.
     */
    private $secret_live;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0
     */
	public function __construct() {
	    $this->access_token_test = !empty(get_option('paypal_access_token_test')) ? get_option('paypal_access_token_test') : '';
	    $this->access_token_live = !empty(get_option('paypal_access_token_live')) ? get_option('paypal_access_token_live') : '';
        $this->client_test = !empty(get_option('paypal_client_test')) ? get_option('paypal_client_test') : '';
        $this->secret_test = !empty(get_option('paypal_secret_test')) ? get_option('paypal_secret_test') : '';
        $this->client_live = !empty(get_option('paypal_client_live')) ? get_option('paypal_client_live') : '';
        $this->secret_live = !empty(get_option('paypal_secret_live')) ? get_option('paypal_secret_live') : '';
	}

    /**
     * Get Paypal account name.
     *
     * @param $mode
     * @return Account
     */
    function get_account_name($mode) {
        return get_option('paypal_account_name_' . $mode);
    }

    /**
     * Get Paypal client_id.
     *
     * @return string
     */
    function get_client_live() {
        return $this->client_live;
    }

    /**
     * Get Paypal client_id.
     *
     * @return string
     */
    function get_client_test() {
        return $this->client_test;
    }

    /**
     * Get Paypal secret key.
     *
     * @return string
     */
    function get_secret_live() {
        return $this->secret_live;
    }

    /**
     * Get Paypal secret key.
     *
     * @return string
     */
    function get_secret_test() {
        return $this->secret_test;
    }

    /**
     * Get Paypal test auth.
     *
     * @return string
     */
    function get_test_auth() {
        return base64_encode($this->client_test . ':' . $this->secret_test);;
    }

    /**
     * Get Paypal live auth.
     *
     * @return string
     */
    function get_live_auth() {
        return base64_encode($this->client_live . ':' . $this->secret_live);;
    }

    /**
     * Set Paypal client_id.
     *
     * @param $value
     * @return string
     */
    function set_client_live($value) {
        $this->client_live = $value;
        update_option('paypal_client_live', $value);
    }

    /**
     * Set Paypal client_id.
     *
     * @param $value
     * @return string
     */
    function set_client_test($value) {
        $this->client_test = $value;
        update_option('paypal_client_test', $value);
    }

    /**
     * Set Paypal secret key.
     *
     * @param $value
     * @return string
     */
    function set_secret_live($value) {
        $this->secret_live = $value;
        update_option('paypal_secret_live', $value);
    }

    /**
     * Set Paypal secret key.
     *
     * @param $value
     * @return string
     */
    function set_secret_test($value) {
        $this->secret_test = $value;
        update_option('paypal_secret_test', $value);
    }

    /**
     * Connect to paypal.
     *
     * @param $mode
     * @return void
     */
    function connect($mode) {

        $product = new North_Checkout_Product();

        $auth = '';

        if($mode === 'live') {
            if (empty($this->client_live) || empty($this->secret_live)) {
                $this->disconnect('live');
                return "Paypal disconnected";
            } else {
                $auth = base64_encode($this->client_live . ':' . $this->secret_live);
            }
        }

        if($mode !== 'live') {
            if(empty($this->client_test) || empty($this->secret_test)) {
                $this->disconnect('test');
                return "Paypal disconnected";
            }else {
                $auth = base64_encode($this->client_test . ':' . $this->secret_test);
            }
        }

        $args = [
            'headers' => [
                'Authorization' => "Basic $auth"
            ],
            'body'    => 'grant_type=client_credentials',
        ];

        $prefix = '';

        if($mode !== 'live') {
            $prefix = '.sandbox';
        }

        $response = wp_remote_post( 'https://api' . $prefix . '.paypal.com/v1/oauth2/token', $args );

        $response_body = wp_remote_retrieve_body( $response );

        $data = json_decode($response_body);

        if(isset($data->error_description)) {
            return $data->error_description;
        }

        if (isset($data->access_token)) {

            if($mode !== 'live') {
                $this->access_token_test = $data->access_token;
            }else {
                $this->access_token_live = $data->access_token;
            }

            update_option('paypal_access_token_' . $mode, $data->access_token);

            $response = wp_remote_get('https://api' . $prefix . '.paypal.com/v1/identity/oauth2/userinfo?schema=paypalv1.1',
                array(
                    'headers' => array(
                        'Authorization' => 'Bearer ' . $data->access_token,
                        'Accept' => 'application/json',
                    ),
                )
            );

            $data = json_decode(wp_remote_retrieve_body( $response ));

            if (isset($data->name)) {
                update_option('paypal_account_name_' . $mode, $data->name);
            }
        }

        $product->sync_products($stripe_sync = false);
    }

    /**
     * Check paypal connection.
     *
     * @param $mode
     * @return bool
     */
    function is_connected($mode) {

        if(!empty(get_option('paypal_access_token_' . $mode))) {
            return get_option('paypal_access_token_' . $mode);
        }

        return false;
    }

    /**
     * Disconnect paypal connection.
     *
     * @param $mode
     * @return void
     */
    function disconnect($mode) {

        if(isset($_POST['mode'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            $mode = sanitize_text_field( $_POST['mode'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing
        }

        delete_option('paypal_access_token_' . $mode);
        delete_option('paypal_account_name_' . $mode);

        if(isset($_POST['mode'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_die();
        }
    }

    /**
     * Create paypal product and plan.
     *
     * @param $mode
     * @param $product_id
     * @param $type
     * @param $data
     * @param bool $need_bump
     * @return bool
     */
    function create_plan($mode, $product_id, $type, $data) {

        do_action('north_before_paypal_plan_created', $mode, $product_id, $data);

        $auth = base64_encode($this->client_live . ':' . $this->secret_live);
        $prefix = '';

        if( $mode === 'test') {
            $auth = base64_encode($this->client_test . ':' . $this->secret_test);
            $prefix = '.sandbox';
        }

        $headers = array(
            'Content-Type' => 'application/json',
            'Authorization' => "Basic $auth",
        );

        $date_modify = date( 'jnYGis' );

        $product_data = array(
            'id' => 'plan_' . $product_id . '_' . $date_modify,
            'name' => get_the_title($product_id),
            "type" => "SERVICE"
        );

        $product_args = array(
            'headers' => $headers,
            'body'    => json_encode($product_data),
        );

        //Create paypal product
        $product_response = json_decode( wp_remote_retrieve_body( wp_remote_post( 'https://api'.$prefix.'.paypal.com/v1/catalogs/products', $product_args ) ) );

        $plan_data = array(
            'product_id' => $product_response->id,
            'name' => $product_response->name,
            'status' => 'active',
            'payment_preferences' => array(
                'setup_fee' => array(
                    'value' =>'0',
                    'currency_code' => North_Checkout_Admin::get_current_currency(),
                )
            )
        );

        $sequence = 1;

        if( $type == 'subscription') {

            if(isset($data['product_trial_count']) && !empty( $data['product_trial_count'])) {

                $interval = $data['product_trial_type'] === 'Days' ? 'DAY' : 'MONTH';
                $plan_data['billing_cycles'][] = array(
                    'frequency' => array(
                        'interval_unit' => $interval,
                        'interval_count' => Clean::int($data['product_trial_count'])
                    ),
                    'tenure_type' => 'TRIAL',
                    'sequence' => $sequence,
                    'total_cycles' => 1,
                    'pricing_scheme' => array(
                        'fixed_price' => array(
                            'value' => "0",
                            'currency_code' => North_Checkout_Admin::get_current_currency(),
                        )
                    )
                );

                $sequence++;
            }

            if(isset($data['product_start_price_sub']) && !empty( $data['product_start_price_sub']) && floatval($data['product_start_price_sub']) > 0) {
                $plan_data['billing_cycles'][] = array(
                    'frequency' => array(
                        'interval_unit' => strtoupper($data['product_frequency_sub']),
                        'interval_count' => 1
                    ),
                    'tenure_type' => 'TRIAL',
                    'sequence' => $sequence,
                    'total_cycles' => 1,
                    'pricing_scheme' => array(
                        'fixed_price' => array(
                            'value' => round(floatval($data['product_start_price_sub']), 2),
                            'currency_code' => North_Checkout_Admin::get_current_currency(),
                        )
                    )
                );

                $sequence++;
            }

            $plan_data['billing_cycles'][] = array(
                'frequency' => array(
                    'interval_unit' => strtoupper($data['product_frequency_sub']),
                    'interval_count' => 1
                ),
                'tenure_type' => 'REGULAR',
                'sequence' => $sequence,
                'total_cycles' => 0,
                'pricing_scheme' => array(
                    'fixed_price' => array(
                        'value' => round(floatval($data['product_price']), 2),
                        'currency_code' => North_Checkout_Admin::get_current_currency(),
                    )
                )
            );

        }else {

            $payment_number = intval($data['product_payment_number']) !== 0 ? intval($data['product_payment_number']) : 1;

            if(isset($data['product_start_price']) && !empty( $data['product_start_price']) && floatval($data['product_start_price']) > 0) {
                $plan_data['billing_cycles'][] = array(
                    'frequency' => array(
                        'interval_unit' => strtoupper($data['product_frequency']),
                        'interval_count' => 1
                    ),
                    'tenure_type' => 'TRIAL',
                    'sequence' => $sequence,
                    'total_cycles' => 1,
                    'pricing_scheme' => array(
                        'fixed_price' => array(
                            'value' => round(floatval($data['product_start_price']), 2),
                            'currency_code' => North_Checkout_Admin::get_current_currency(),
                        )
                    )
                );

                $sequence++;
            }

            $plan_data['billing_cycles'][] = array(
                'frequency' => array(
                    'interval_unit' => strtoupper($data['product_frequency']),
                    'interval_count' => 1
                ),
                'tenure_type' => 'REGULAR',
                'sequence' => $sequence,
                'total_cycles' => Clean::int($payment_number),
                'pricing_scheme' => array(
                    'fixed_price' => array(
                        'value' => round(((floatval($data['product_price'])) - floatval($data['product_start_price'])) / $payment_number, 2),
                        'currency_code' => North_Checkout_Admin::get_current_currency(),
                    )
                )
            );
        }

        $plan_args = array(
            'headers' => $headers,
            'body'    => json_encode($plan_data),
        );

        $response = json_decode( wp_remote_retrieve_body( wp_remote_post( 'https://api'.$prefix.'.paypal.com/v1/billing/plans', $plan_args ) ) );

        if(isset($response->id)) {
            $return_array = array(
                'success' => true,
                'product_id' => $response->id,
                'product_plan_id' => $product_response->id,

            );

            do_action('north_after_paypal_plan_created', $mode, $product_id, $data);

            return $return_array;
        }

        do_action('north_failed_paypal_plan_created', $mode, $product_id, $data);

        return false;
    }

    function recreate_plans() {
        $args = array(
            'numberposts' => -1,
            'post_type'   => 'north_product',
            'post_status' => 'any',
            'meta_query' => array(
                'product_type' => array(
                    'key'     => 'product_type',
                    'value'   => array('Subscription', 'Payment Plan'),
                    'compare' => 'IN',
                ),
            ),
        );

        $products = get_posts( $args );

        if($this->is_connected('test') || $this->is_connected('live')) {
            foreach ($products as $product) {

                $data = get_post_meta($product->ID);
                $new_data = array();

                foreach ($data as $key => $value) {
                    $new_data[$key] = $value[0];
                }

                if($this->is_connected('test')) {
                    $paypal_response_test = $this->create_plan('test', $product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data);
                }

                if($this->is_connected('live')) {
                    $paypal_response_live = $this->create_plan('live', $product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data);
                }

                if($paypal_response_test || $paypal_response_live) {
                    $paypal_response = true;
                }

                if($paypal_response_test) {
                    update_post_meta($product->ID, 'paypal_test_product_id', $paypal_response_test['product_id']);
                    update_post_meta($product->ID, 'paypal_test_product_plan_id', $paypal_response_test['product_plan_id']);
                }

                if($paypal_response_live) {
                    update_post_meta($product->ID, 'paypal_live_product_id', $paypal_response_live['product_id']);
                    update_post_meta($product->ID, 'paypal_live_product_plan_id', $paypal_response_test['product_plan_id']);
                }

                if($paypal_response) {
                    update_post_meta($product->ID, 'paypal_active', 'active');
                }
            }
        }
    }

    /**
     * Delete paypal product and plan.
     *
     * @param $product_id
     * @return void
     */
    function delete_plan($mode, $product_id) {

        do_action('north_before_paypal_plan_deleted', $mode, $product_id);

        $auth = base64_encode($this->client_live . ':' . $this->secret_live);
        $prefix = '';

        if( $mode === 'test') {
            $auth = base64_encode($this->client_test . ':' . $this->secret_test);
            $prefix = '.sandbox';
        }

        $headers = array(
            'Content-Type' => 'application/json',
            'Authorization' => "Basic $auth",
        );

        $args = array(
            'method' => 'POST',
            'headers' => $headers,
        );

        $plan_id = get_post_meta($product_id, 'paypal_'.$mode.'_product_id', true);
        //$plan_bump_id = get_post_meta($product_id, 'paypal_bump_id', true);

        $response = wp_remote_request('https://api'.$prefix.'.paypal.com/v1/billing/plans/' . $plan_id . '/deactivate', $args);
        //$response = wp_remote_request('https://api.sandbox.paypal.com/v1/billing/plans/' . $plan_bump_id . '/deactivate', $args);

        do_action('north_after_paypal_plan_deleted', $mode, $product_id);

    }

    /**
     * Get paypal current mode.
     *
     * @return void
     */
    function current_mode() {

        if( !empty(get_option('test_paypal')) ){
            return 'test';
        }

        return 'live';

    }

}
