<?php
/**
 * Product class.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Product {

    /**
     * Product id.
     *
     * @access   private
     * @var      integer    $product_id    product id.
     */
    private $product_id;

    /**
     * Query products object.
     *
     * @access   private
     * @var      object    $product_query    Query products array.
     */
    private $product_query;

    function __construct($product_id = null) {
        $this->product_id = $product_id;
    }

    /**
     * Register Product Custom Post Type with settings.
     *
     */
    function register_product() {

        register_post_type( 'north_product', [
            'label'  => null,
            'labels' => [
                'name'               => 'Product',
                'singular_name'      => 'Product',
            ],
            'public'              => true,
            'show_in_menu'        => false,
            'show_in_rest'        => true,
            'hierarchical'        => false,
            'supports'            => [ 'title', 'editor', 'custom-fields'],
            'rewrite'             => true,
            'query_var'           => true,
        ] );

        register_post_status( 'active', array(
            'label'                     => _x( 'Active', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_product'),
            'label_count'               => _n_noop( 'Active <span class="count">(%s)</span>', 'Active <span class="count">(%s)</span>' ),
        ) );

        register_post_status( 'not_active', array(
            'label'                     => _x( 'Not Active', 'order' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'post_type'                 => array( 'north_product'),
            'label_count'               => _n_noop( 'Not active <span class="count">(%s)</span>', 'Not active <span class="count">(%s)</span>' ),
        ) );
    }

    /**
     * Prepare order to transfer to Zapier.
     */
    function product_prepare_zap($product) {

        $product_obj = new North_Checkout_Product($product->ID);
        $product_type = get_post_meta($product->ID, 'product_type', true);
        $variations = $product_obj->get_variations();
        $variation_string = null;

        if( $variations && $variations !== '[]') {

            foreach($variations as $variation) {

                $values = array();

                foreach($variation["values"] as $value) {
                    $values[] = isset($value["variation_attribute"]) ? $value["variation_attribute"] : '';
                }

                $variation_string .= $variation["variation_name"] . " ( " . implode(', ', $values) . " ) ";
            }

        }

        $product_data = array(
            'id' => $product->ID,
            'title' => $product->post_title,
            'slug' => $product->post_name,
            'status' => $product->post_status,
            'date_created' => $product->post_date,
            'date_modified' => $product->post_modified,
            'product_type' => $product_type,
            'product_price' => strval($product_obj->get_price()),
            'product_currency' => strval(North_Checkout_Admin::get_current_currency()),
            'product_currency_symb' => North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency()),
            'product_quantity' => strval($product_obj->get_quantity()),
            'bump_reload' => strval($product_obj->reload_bump()),
            'is_variable' => strval($product_obj->is_variable()),
            'product_frequency' => strval($product_obj->get_frequency()),
            'product_payment_number' => strval($product_obj->get_payment_count()),
            'product_start_price' => strval($product_obj->get_start_price()),
            'product_trial_count' => strval($product_obj->get_trial_count()),
            'product_trial_type' => strval($product_obj->get_trial_type()),
            'product_variation' => $variation_string,
        );

        return apply_filters('north_prepare_zapier_product', $product_data);

    }

    /**
     * Register route for Order.
     */
    function register_product_route() {

        register_rest_route( 'north-checkout/v1', '/products/', array(
            'methods'  => 'GET',
            'callback' => array($this, 'get_rest_products'),
            'permission_callback' => '__return_true'
        ) );
    }

    /**
     * Register route for Product.
     */
    function get_rest_products( WP_REST_Request $request ) {

        $products = get_posts( array(
            'numberposts' => -1,
            'post_type'   => 'north_product',
            'post_status' => 'any',
            'suppress_filters' => true,
        ) );

        $product_data = array();

        foreach ($products as $product) {
            $product_data[] = $this->product_prepare_zap($product);
        }

        $response = rest_ensure_response( $product_data );
        $response->set_status( 200 );
        return $response;
    }

    /**
     * Function for including templates.
     *
     * @since    1.0
     *
     * @param $template string  name of template without .php
     * @param $dir string  name of template folder
     * @param $args array  arguments for template
     * @param $template_path string  path to plugin folder (by default is plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/)
     */
    public function template( $template, $dir = '', $args = array(), $template_path = '' ) {
        // @TODO: Refactor to remove extract().
        extract( $args, EXTR_OVERWRITE ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract
        $dir       = ! empty( $dir ) ? trailingslashit( $dir ) : $dir;
        $base_path = ! empty( $template_path ) ? $template_path : plugin_dir_path( dirname( __FILE__ ) ) . 'admin/templates/';

        $path = $base_path . $dir . $template . '.php';
        include $path;
    }

    /**
     * Get single product by id.
     *
     * @since   1.0
     *
     * @param   $product_id integer Order Id
     *
     * @return array
     */
    function get_product($product_id) {

        return get_post_meta($product_id);

    }

    /**
     * Send hook to zapier.
     *
     * @since   1.0
     *
     * @param   $post_id integer Order Id
     *
     */
    function send_hook($post_id) {

        $hooks = get_option('north_hooks');

        if ($hooks) {
            $product = get_post($post_id);
            $product_data = $this->product_prepare_zap($product);

            foreach ($hooks as $hook) {

                if (isset($hook["hook_type"]) && $hook["hook_type"] === 'product_created' && isset($hook["hook"])) {
                    wp_remote_post($hook["hook"], array(
                            'method' => 'POST',
                            'timeout' => 45,
                            'redirection' => 5,
                            'headers' => array(),
                            'body' => $product_data,
                        )
                    );
                }
            }
        }
    }

    /**
     * Create new product.
     *
     * @param $product_name
     * @param string $status Product status slug ( active | not_active)
     * @param array $data Array of product meta
     * @return int|WP_Error Product id or WP_Error
     */
    function create_product($product_name, $status = 'active', $data) {

        do_action('north_before_product_created', $product_name, $data, $status);

        $paypal = new North_Checkout_Paypal();
        $stripe = new North_Checkout_Stripe();

        $product_data = array(
            'post_type'     => 'north_product',
            'post_title'    => apply_filters('north_create_product_name', wp_strip_all_tags( $product_name )),
            'post_status'   => $status
        );

        $need_bump = false;
        $stripe_response = false;
        $paypal_response = false;
        $paypal_response_test = false;
        $paypal_response_live = false;

        if($data['reload_bump']) {
            $need_bump = true;
        }

        $product_id = wp_insert_post( $product_data );

        $data = apply_filters('north_create_product_data', $data, $product_id);

        // If product type == Subscription then creating plan for subscription in Stripe
        if( isset($data['product_type']) && ($data['product_type'] === 'Subscription' || $data['product_type'] === 'Payment Plan')) {

            if($stripe->is_connected()) {
                $stripe_response = $stripe->create_plan($product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data, $need_bump);
            }

            if($paypal->is_connected('test')) {
                $paypal_response_test = $paypal->create_plan('test', $product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data);
            }

            if($paypal->is_connected('live')) {
                $paypal_response_live = $paypal->create_plan('live', $product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data);
            }

            if($paypal_response_test || $paypal_response_live) {
                $paypal_response = true;
            }

            if($stripe_response || $paypal_response) {
                $products_count = wp_count_posts( 'north_product' );
                $product_number = intval($products_count->active) + intval($products_count->not_active) + intval($products_count->draft) + intval($products_count->pending) + 1;
                update_post_meta($product_id, 'product_number', $product_number);

                foreach($data as $key => $value) {
                    update_post_meta($product_id, $key, $value);
                }
            }

            if($paypal_response_test) {
                update_post_meta($product_id, 'paypal_test_product_id', $paypal_response_test['product_id']);
                update_post_meta($product_id, 'paypal_test_product_plan_id', $paypal_response_test['product_plan_id']);
            }

            if($paypal_response_live) {
                update_post_meta($product_id, 'paypal_live_product_id', $paypal_response_live['product_id']);
                update_post_meta($product_id, 'paypal_live_product_plan_id', $paypal_response_test['product_plan_id']);
            }

            if($stripe_response) {
                update_post_meta($product_id, 'stripe_active', 'active');
            }

            if($paypal_response) {
                update_post_meta($product_id, 'paypal_active', 'active');
            }

            if(!$stripe_response && !$paypal_response ) {
                wp_delete_post($product_id);
            }

        }else {

            foreach($data as $key => $value) {
                update_post_meta($product_id, $key, $value);
            }

            if($this->is_variable($product_id)) {
                $price = '';
                $variations = $this->get_variations($product_id);

                foreach ($variations as $variation) {
                    if (isset($variation['variation_primary']) && $variation['variation_primary'][0] === 'on') {
                        $price = $variation["values"][0]['variation_price'];
                        update_post_meta($product_id, 'product_price', $price);
                    }
                }

            }

        }

        if ( is_wp_error($product_id) ) {
            do_action('north_product_created_failed', $product_id, $data);
        }else {
            do_action('north_product_created_success', $product_id, $data);
        }

        return $product_id;
    }

    /**
     * Update product.
     *
     * @param $product_name
     * @param string $status Product status slug ( active | not_active)
     * @param array $data Array of product meta
     * @return int|WP_Error Product id or WP_Error
     */
    function update_product($product_id = null, $product_name, $status = 'active', $data) {

        do_action('north_before_product_updated', $product_id, $product_name, $data, $status);

        $stripe = new North_Checkout_Stripe();
        $paypal = new North_Checkout_Paypal();

        $product_id = !empty($product_id) ? $product_id : $this->product_id;

        $product_data = array(
            'ID'            => $product_id,
            'post_title'    => wp_strip_all_tags( $product_name ),
            'post_status'   => $status
        );

        $need_bump = false;

        if($data['reload_bump']) {
            $need_bump = true;
        }

        $data = apply_filters('north_update_product_data', $data, $product_id);

        // If product type == Subscription then creating plan for subscription in Stripe
        if( isset($data['product_type']) && ($data['product_type'] === 'Subscription' || $data['product_type'] === 'Payment Plan')) {

            if ($stripe->is_connected()) {
                $stripe->delete_plan($product_id);
                $stripe_response = $stripe->create_plan($product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data, $need_bump);
            }

            if($paypal->is_connected('test')) {
                $paypal_response_test = $paypal->create_plan('test', $product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data);
            }

            if($paypal->is_connected('live')) {
                $paypal_response_live = $paypal->create_plan('live', $product_id, strtolower(str_replace(" ", "_", $data["product_type"])), $data);
            }

            if($paypal_response_test || $paypal_response_live) {
                $paypal_response = true;
            }

            if ($stripe_response || $paypal_response) {
                wp_update_post($product_data);

                foreach ($data as $key => $value) {
                    update_post_meta($product_id, $key, $value);
                }
            }

            if($paypal_response_test) {
                update_post_meta($product_id, 'paypal_test_product_id', $paypal_response_test['product_id']);
                update_post_meta($product_id, 'paypal_test_product_plan_id', $paypal_response_test['product_plan_id']);
            }

            if($paypal_response_live) {
                update_post_meta($product_id, 'paypal_live_product_id', $paypal_response_live['product_id']);
                update_post_meta($product_id, 'paypal_live_product_plan_id', $paypal_response_test['product_plan_id']);
            }

            if($stripe_response) {
                update_post_meta($product_id, 'stripe_active', 'active');
            }

            if($paypal_response) {
                update_post_meta($product_id, 'paypal_active', 'active');
            }

        }else {

            wp_update_post( $product_data );

            foreach($data as $key => $value) {
                update_post_meta($product_id, $key, $value);
            }

            if ($this->is_variable($product_id)) {
                $price = '';
                $variations = $this->get_variations($product_id);

                foreach ($variations as $variation) {
                    if (isset($variation['variation_primary']) && $variation['variation_primary'][0] === 'on') {
                        $price = $variation["values"][0]['variation_price'];
                        update_post_meta($product_id, 'product_price', $price);
                    }
                }
            }
        }

        do_action('north_product_updated_success', $product_id, $data);

        return $product_id;
    }

    /**
     * Delete product.
     *
     * @param integer $product_id Product id
     * @return false|WP_Post|null
     */
    function delete_product($product_id) {

        $stripe = new North_Checkout_Stripe();
        $paypal = new North_Checkout_Paypal();

        $product_id = isset($_POST['delete_id']) ? sanitize_text_field($_POST['delete_id']) : $product_id; // phpcs:ignore WordPress.Security.NonceVerification.Missing

        do_action('north_before_product_deleted', $product_id);

        $product = new North_Checkout_Product($product_id);

        if( $product->get_type() === 'Subscription' || $product->get_type() === 'Payment Plan' ) {
            $stripe->delete_plan($product_id);

            if( $paypal->is_connected('test')) {
                $paypal->delete_plan('test', $product_id);
            }

            if( $paypal->is_connected('live')) {
                $paypal->delete_plan('live', $product_id);
            }
        }

        if (isset($_POST['delete_id'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_delete_post($product_id);
            wp_die();
        }

        $result = wp_delete_post($product_id);

        do_action('north_after_product_deleted', $product_id);

        return $result;
    }

    /**
     * Get product loop.
     *
     * @param int $per_page
     * @param bool $is_array
     * @param array $meta
     * @return array
     */
    function get_products( $per_page = 8, $is_array = false, $meta = null, $compare = null) {

        $paged = isset($_POST['paged']) ? sanitize_text_field($_POST['paged']) : 1;  // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $posts_per_page = isset($_POST['posts_per_page']) ? sanitize_text_field($_POST['posts_per_page']) : $per_page; // phpcs:ignore WordPress.Security.NonceVerification.Missing
        $meta_query = array();

        if($meta) {
            foreach ($meta as $key => $value) {
                $meta_query[] = array(
                    'key' => $key,
                    'value' => $value,
                    'compare' => $compare,
                );
            }
        }

        $args = array(
            'post_type'         => 'north_product',
            'posts_per_page'    => $posts_per_page,
            'paged'             => $paged,
            'meta_query'        => $meta_query
        );

        $args = apply_filters('north_get_products', $args);

        $this->product_query = new WP_Query( $args );

        if($this->product_query->have_posts()) {

            if($is_array) {

                $result = array();

                while($this->product_query->have_posts()) : $this->product_query->the_post();

                    $result[get_the_ID()] = esc_html__( get_the_title(), 'north-checkout' );

                endwhile;

                wp_reset_query();

                $result['0'] = esc_html__('Select Product', 'north-checkout' );

                return $result;

            }else {

                while($this->product_query->have_posts()) : $this->product_query->the_post();

                    $product_id = get_the_ID();

                    $args = array(
                        'product_price' => $this->get_price_html($product_id),
                        'paypal_active' => get_post_meta($product_id, 'paypal_active', true),
                        'stripe_active' => get_post_meta($product_id, 'stripe_active', true),
                    );

                    if( get_post_meta($product_id, 'reload_bump', true) == 'on') {
                        $args['bump_price'] = $this->get_bump_price_html($product_id);
                    }

                    $args['product_type'] = $this->get_type();

                    $this->template('single-product','product', $args);

                endwhile;

                wp_reset_query();

            }

        }else {
            if( isset($_POST['paged'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
                echo 'error';
            }elseif( $is_array ){
                return;
            }else {
                return 0;
            }
        }

        if( isset($_POST['posts_per_page']) ) // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_die();

    }

    /**
     * Get order loop pagination.
     */
    function get_pagination() {
        if ($this->product_query->max_num_pages > 1) {
            $this->template('product-pagination','product');
        }
    }

    /**
     * Get product id.
     *
     * @param null $product_id
     *
     * @return false|int|null
     */
    function get_id($product_id = null) {
        if (isset($product_id)) {
            return $product_id;
        }elseif (isset($this->product_id)){
            return $this->product_id;
        }else {
            return get_the_ID();
        }
    }

    /**
     * Get product price.
     *
     * @param $product_id
     * @param bool $without_currency
     * @param string $currency_position
     *
     * @return string product price with currency
     */
    function get_price($product_id = null, $without_currency = true, $currency_position = 'right') {

        $id = $this->get_id($product_id);
        $price = '';

        if($without_currency) {
            $price = floatval( get_post_meta( $id, 'product_price', true ) );
        }else {
            if($currency_position) {
                $price = get_post_meta($id, 'product_price', true) . North_Checkout_Admin::get_current_currency();
            }else {
                $price = North_Checkout_Admin::get_current_currency() . get_post_meta($id, 'product_price', true);
            }
        }

        return apply_filters('north_get_price', $price, $product_id, $without_currency, $currency_position);
    }

    /**
     * Get bump price.
     *
     * @param $product_id
     * @param bool $without_currency
     * @param string $currency_position
     *
     * @return string product price with currency
     */
    function get_bump_price($product_id = null, $without_currency = true, $currency_position = 'right') {

        $id = $this->get_id($product_id);
        $price = '';

        if($without_currency) {
            $price = floatval( get_post_meta( $id, 'bump_price', true ) );
        }else {
            if($currency_position) {
                $price = get_post_meta($id, 'bump_price', true) . North_Checkout_Admin::get_current_currency();
            }else {
                $price = North_Checkout_Admin::get_current_currency() . get_post_meta($id, 'bump_price', true);
            }
        }

        return apply_filters('north_get_bump_price', $price, $product_id, $without_currency, $currency_position);
    }

    /**
     * Get raw product price.
     *
     * @param $product_id
     *
     * @return string product price without currency and html (crop of get_price_html())
     */
    function get_raw_price($product_id = null) {

        $product_id = $this->get_id($product_id);

        $product_type = $this->get_type($product_id);
        $price = floatval(get_post_meta($product_id, 'product_price', true));

        $start_price = $this->get_start_price($product_id);

        $payment_number = intval($this->get_payment_count($product_id)) !== 0 ? intval($this->get_payment_count($product_id)) : 1;

        if( !empty($payment_number) ) {
            $price = $price / $payment_number;
        }

        if( !empty($start_price) ) {
            $price = $start_price;
        }

        return apply_filters('north_get_raw_price', $price, $product_id);

    }

    /**
     * Get product price html.
     *
     * @param $product_id
     *
     * @return string product price with currency and html
     */
    function get_price_html($product_id = null) {

        $product_id = $this->get_id($product_id);

        $product_type = $this->get_type($product_id);
        $price = floatval(get_post_meta($product_id, 'product_price', true));

        $trial = $this->get_trial_count($product_id);
        $trial_count = $trial ? ' | Free trial: ' . $trial . ' ' . $this->get_trial_type($product_id) : '';
        $start_price = $this->get_start_price($product_id);
        $start_price_html = $start_price  ? ' | First payment : ' . North_Checkout_Admin::formatted_price($start_price) . '</span>' : '';
        $subscription_price = North_Checkout_Admin::formatted_price($price, false);


        if( empty($start_price_html) ) {
            $subscription_price = North_Checkout_Admin::formatted_price($price);
        }

        $payment_number = intval($this->get_payment_count($product_id)) !== 0 ? intval($this->get_payment_count($product_id)) : 1;
        $price_html = null;

        if($product_type === 'Payment Plan') {

            $one_payment = $price / $payment_number;
            if($start_price) {
                $one_payment = ($price - $start_price) / $payment_number;
                $price_html = 'Price: ' . North_Checkout_Admin::formatted_price($price, false) . ' Total / ' . $payment_number . ' payments of ' . North_Checkout_Admin::formatted_price($one_payment, false) . '</span>' . $start_price_html;
            }else {
                $price_html = 'Price: ' . North_Checkout_Admin::formatted_price($price, false) . ' Total / ' . $payment_number . ' payments of ' . North_Checkout_Admin::formatted_price($one_payment) . '</span>' . $start_price_html;
            }

        }elseif($product_type === 'Subscription') {
            $price_html = '<span class="frequency">' . $this->get_frequency($product_id) . 'ly</span> price: ' . $subscription_price . $start_price_html . $trial_count;
        }else {
            $price_html = 'Price: ' . North_Checkout_Admin::formatted_price($price) . '</span>';
        }

        return apply_filters('north_get_price_html', $price_html, $product_id, $product_type);

    }

    /**
     * Get bump price html.
     *
     * @param $product_id
     *
     * @return string product price with currency and html
     */
    function get_bump_price_html($product_id = null) {

        $id = $this->get_id($product_id);

        $product_type = $this->get_type($id);
        $price = floatval(get_post_meta($id, 'bump_price', true));

        $trial = $this->get_bump_trial_count($id);
        $trial_count = $trial ? ' | Free trial: ' . $trial . ' ' . $this->get_bump_trial_type($id) : '';
        $start_price = $this->get_bump_start_price($id);
        $start_price_html = $start_price  ? ' | First payment : ' . North_Checkout_Admin::formatted_price($start_price) . '</span>' : '';
        $subscription_price = North_Checkout_Admin::formatted_price($price, false);

        if( empty($start_price_html) ) {
            $subscription_price = North_Checkout_Admin::formatted_price($price);
        }

        $payment_number = intval($this->get_bump_payment_count($id)) !== 0 ? intval($this->get_bump_payment_count($id)) : 1;
        $price_html = null;

        if($product_type === 'Payment Plan') {

            $one_payment = $price / $payment_number;
            if($start_price) {
                $one_payment = ($price - $start_price) / $payment_number;
            }
            $price_html = 'Price: ' .  North_Checkout_Admin::formatted_price($price, false) . ' Total / ' . $this->get_bump_payment_count($id) . ' payments of ' .  North_Checkout_Admin::formatted_price($one_payment) . '</span>' . $start_price_html;
        }elseif($product_type === 'Subscription') {
            $price_html = '<span class="frequency">' . $this->get_bump_frequency($id) . 'ly</span> price: ' . $subscription_price . $start_price_html . $trial_count;
        }else {
            $price_html = 'Price: ' . North_Checkout_Admin::formatted_price($price) . '</span>';
        }

        return apply_filters('north_get_bump_price_html', $price_html, $product_id, $product_type);

    }

    /**
     * Get product quantity.
     *
     * @param $product_id
     * @return string product quantity
     */
    function get_quantity($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'product_quantity', true);
    }

    /**
     * Get bump quantity.
     *
     * @param $product_id
     * @return string product quantity
     */
    function get_bump_quantity($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'bump_quantity', true);
    }

    /**
     * Get product type.
     *
     * @param integer $product_id
     *
     * @return string product type
     */
    function get_type($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'product_type', true);
    }

    /**
     * Get product trial count of days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return integer product trial count
     */
    function get_trial_count($product_id = null) {

        $id = $this->get_id($product_id);

        return intval( get_post_meta( $id, 'product_trial_count', true ) );
    }

    /**
     * Get bump trial count of days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return integer product trial count
     */
    function get_bump_trial_count($product_id = null) {

        $id = $this->get_id($product_id);

        return intval( get_post_meta( $id, 'bump_trial_count', true ) );
    }

    /**
     * Get product trial count of days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return integer product trial count
     */
    function get_payment_count($product_id = null) {

        $id = $this->get_id($product_id);

        return intval( get_post_meta( $id, 'product_payment_number', true ) );
    }

    /**
     * Get bumpo trial count of days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return integer product trial count
     */
    function get_bump_payment_count($product_id = null) {

        $id = $this->get_id($product_id);

        return intval( get_post_meta( $id, 'bump_payment_number', true ) );
    }

    /**
     * Get product trial type days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return string product trial type
     */
    function get_trial_type($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'product_trial_type', true);
    }

    /**
     * Get bump trial type days | weeks | months |years.
     *
     * @param integer $product_id
     *
     * @return string product trial type
     */
    function get_bump_trial_type($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'bump_trial_type', true);
    }

    /**
     * Get product frequency.
     *
     * @param integer $product_id
     *
     * @return string product frequency
     */
    function get_frequency($product_id = null) {

        $id = $this->get_id($product_id);

        $type = $this->get_type($id);

        if($type == 'Subscription') {
            return get_post_meta($id, 'product_frequency_sub', true);
        }else {
            return get_post_meta($id, 'product_frequency', true);
        }
    }

    /**
     * Get bump frequency.
     *
     * @param integer $product_id
     *
     * @return string product frequency
     */
    function get_bump_frequency($product_id = null) {

        $id = $this->get_id($product_id);

        $type = $this->get_type($id);

        if($type == 'Subscription') {
            return get_post_meta($id, 'bump_frequency_sub', true);
        }else {
            return get_post_meta($id, 'bump_frequency', true);
        }
    }

    /**
     * Get product variations.
     *
     * @param integer $product_id
     *
     * @return string product variations
     */
    function get_variations($product_id = null) {

        $id = $this->get_id($product_id);

        return get_post_meta($id, 'variations', true);
    }

    /**
     * Get product start price for first day | week | month |year.
     *
     * @param integer $product_id
     * @return float product start price
     */
    function get_start_price($product_id = null) {

        $id = $this->get_id($product_id);

        $type = $this->get_type($id);

        if($type == 'Subscription') {
            if($price = get_post_meta( $id, 'product_start_price_sub', true )) {
                return floatval($price);
            }else {
                return false;
            }
        }else {
            if($price = get_post_meta( $id, 'product_start_price', true )) {
                return floatval($price);
            }else {
                return false;
            }
        }
    }

    /**
     * Get bump start price for first day | week | month |year.
     *
     * @param integer $product_id
     * @return float product start price
     */
    function get_bump_start_price($product_id = null) {

        $id = $this->get_id($product_id);

        $type = $this->get_type($id);

        if($type == 'Subscription') {
            if($price = get_post_meta( $id, 'bump_start_price_sub', true )) {
                return floatval($price);
            }else {
                return false;
            }
        }else {
            if($price = get_post_meta( $id, 'bump_start_price', true )) {
                return floatval($price);
            }else {
                return false;
            }
        }
    }

    /**
     * Get product variation price by variation name.
     *
     * @param integer $product_id
     *
     * @param $current_variations
     * @return bool|float product variation price
     */
    function get_variation_price($product_id = null, $current_variations) {

        $variations = $this->get_variations();
        foreach($variations as $variation) {
            if (isset($variation['variation_primary']) && $variation['variation_primary'][0] === 'on') {
                $current_value = $current_variations[strtolower( str_replace(" ","_", $variation["variation_name"] ) )];
                foreach($variation["values"] as $value) {

                    if($value["variation_attribute"] === $current_value) {
                        return apply_filters('north_get_variation_price', floatval($value["variation_price"]), $product_id, $current_variations);
                    }
                }
            }
        }

        return false;
    }

    /**
     * If pruduct reloaded bump.
     *
     * @param integer $product_id
     *
     * @return bool reload_bump
     */
    function reload_bump($product_id = null) {

        $id = $this->get_id($product_id);

        if( get_post_meta($id, 'reload_bump', true) == 'on')
            return true;
        else
            return false;
    }

    /**
     * If pruduct is variable.
     *
     * @param integer $product_id
     * @return bool is_variable
     */
    function is_variable($product_id = null) {

        $id = $this->get_id($product_id);

        if( get_post_meta($id, 'is_variable', true) == 'on')
            return true;
        else
            return false;
    }

    /**
     * Convert currency code to symbol.
     *
     * @param string $cur_code
     * @return string currency symbol
     */
    function convert_code($cur_code) {

        if(strtolower($cur_code) === 'usd')
            return '$';
        elseif(strtolower($cur_code) === 'eur')
            return '€';
        elseif(strtolower($cur_code) === 'cad')
            return '$';
        elseif(strtolower($cur_code) === 'aud')
            return '$';
        elseif(strtolower($cur_code) === 'gbp')
            return '£';
        elseif(strtolower($cur_code) === 'dkk')
            return 'kr.';
        else 'ERR';

    }

    /**
     * Sync products with stripe and paypal.
     *
     * @param bool $stripe_sync
     * @param bool $paypal_sync
     * @return void
     */
    function sync_products($stripe_sync = true, $paypal_sync = true) {

        do_action('north_before_sync_products', $stripe_sync, $paypal_sync);

        $args = array(
            'numberposts' => -1,
            'post_type'   => 'north_product',
            'post_status' => 'any',
            'meta_query' => array(
                'product_type' => array(
                    'key'     => 'product_type',
                    'value'   => array('Subscription', 'Payment Plan'),
                    'compare' => 'IN',
                ),
            ),
        );

        if(isset($_POST['ajax_response'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            $stripe_sync = true;
            $paypal_sync = true;
        }

        $products = get_posts( $args );
        $stripe = null;
        $paypal = null;

        $stripe = new North_Checkout_Stripe();
        $paypal = new North_Checkout_Paypal();

        if($stripe_sync && $stripe->is_connected()) {

            if( !empty( get_option('stripe_private_key') ) ) {
                try {
                    $stripe_client = new \NorthStripe\StripeClient(get_option('stripe_private_key'));
                    $current_plans_live = $stripe_client->plans->all(['limit' => 100])["data"];
                    $plan_array_live = array_column($current_plans_live,'id');

                    foreach ($products as $product) {

                        $data = get_post_meta($product->ID);
                        $new_data = array();

                        foreach($data as $key => $value) {
                            $new_data[$key] = $value[0];
                        }


                        if (!in_array('plan_' . $product->ID, $plan_array_live)) {

                            if($data['reload_bump']) {
                                $need_bump = true;
                            }

                            $stripe->create_plan($product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data, $need_bump);
                        }

                        update_post_meta($product->ID, 'stripe_active', 'active');
                    }

                }catch (Stripe_Error $e) {} catch (Exception $e) {}
            }

            if( !empty( get_option('stripe_test_private_key') ) ) {
                try {
                    $stripe_client = new \NorthStripe\StripeClient(get_option('stripe_test_private_key'));
                    $current_plans_test = $stripe_client->plans->all(['limit' => 100])["data"];
                    $plan_array_test = array_column($current_plans_test,'id');

                    foreach ($products as $product) {

                        $data = get_post_meta($product->ID);
                        $new_data = array();

                        foreach($data as $key => $value) {
                            $new_data[$key] = $value[0];
                        }

                        if (!in_array('plan_' . $product->ID, $plan_array_test)) {

                            if($data['reload_bump']) {
                                $need_bump = true;
                            }

                            $stripe->create_plan($product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data, $need_bump);

                        }

                        update_post_meta($product->ID, 'stripe_active', 'active');
                    }

                }catch (Stripe_Error $e) {} catch (Exception $e) {}
            }
        }

        if($paypal_sync && $paypal->is_connected('test')) {
            $current_page = 1;
            $total_pages = 1;
            $paypal_products = array();

            do {

                $response = wp_remote_get('https://api.sandbox.paypal.com/v1/catalogs/products?page_size=20&page=' . $current_page . '&total_required=true',
                    array(
                        'headers' => array(
                            'Authorization' => 'Basic ' . $paypal->get_test_auth(),
                            'Accept' => 'application/json',
                        ),
                    )
                );

                $data = json_decode(wp_remote_retrieve_body($response));

                $total_pages = intval($data->total_pages);
                $paypal_products = array_merge($paypal_products, $data->products);

                $current_page++;

            } while ($current_page != $total_pages + 1);

            $plan_array_test = array_column($paypal_products, 'id');

            foreach ($products as $product) {

                $data = get_post_meta($product->ID);

                $new_data = array();

                foreach($data as $key => $value) {
                    $new_data[$key] = $value[0];
                }

                if (!in_array('plan_' . $product->ID, $plan_array_test)) {

                    $response = $paypal->create_plan('test', $product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data);

                    update_post_meta($product->ID, 'paypal_test_product_id', $response['product_id']);
                    update_post_meta($product->ID, 'paypal_test_product_plan_id', $response['product_plan_id']);
                }

                update_post_meta($product->ID, 'paypal_active', 'active');
            }

        }

        if($paypal_sync && $paypal->is_connected('live')) {
            $current_page = 1;
            $total_pages = 1;
            $paypal_products = array();

            do {

                $response = wp_remote_get('https://api.paypal.com/v1/catalogs/products?page_size=20&page=' . $current_page . '&total_required=true',
                    array(
                        'headers' => array(
                            'Authorization' => 'Basic ' . $paypal->get_live_auth(),
                            'Accept' => 'application/json',
                        ),
                    )
                );

                $data = json_decode(wp_remote_retrieve_body($response));

                $total_pages = intval($data->total_pages);
                $paypal_products = array_merge($paypal_products, $data->products);

                $current_page++;

            } while ($current_page != $total_pages + 1);

            $plan_array_live = array_column($paypal_products, 'id');

            foreach ($products as $product) {

                $data = get_post_meta($product->ID);
                $new_data = array();

                foreach($data as $key => $value) {
                    $new_data[$key] = $value[0];
                }

                if (!in_array('plan_' . $product->ID, $plan_array_live)) {

                    if ($data['reload_bump']) {
                        $need_bump = true;
                    }

                    $response = $paypal->create_plan('live', $product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data);

                    update_post_meta($product->ID, 'paypal_live_product_id', $response['product_id']);
                    update_post_meta($product->ID, 'paypal_live_product_plan_id', $response['product_plan_id']);

                }

                update_post_meta($product->ID, 'paypal_active', 'active');
            }

        }

        do_action('north_after_sync_products', $stripe_sync, $paypal_sync);


        if(isset($_POST['ajax_response'])) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
            wp_die();
        }
    }
}
