<?php

use NorthStripe\Account;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the Stripe
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/admin
 * @author     North Plugins <hello@northcheckout.com>
 */

class North_Checkout_Stripe {

    /**
     * Stripe privat key.
     *
     * @access   private
     * @var      string    $privat_key    $privat key.
     */
    private $private_key;

    /**
     * Stripe client.
     *
     * @access   private
     * @var      string    $stripe    stripe client object.
     */
    private $stripe;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0
     */
    public function __construct() {
        $this->private_key = get_option('stripe_private_active');
    }

    /**
     * Create payment plans in stripe.
     *
     * @param $product_id
     * @param $data
     * @param bool $need_bump
     * @return bool
     * @throws \NorthStripe\Exception\ApiErrorException
     * @throws \NorthStripe\Exception\InvalidArgumentException
     * @since   1.0
     */
    function create_plan($product_id, $type = 'subscription', $data, $need_bump = false) {

        do_action('north_before_stripe_plan_created', $product_id, $data, $need_bump);

        try {

            $args = array();
            $args_bump = array();
            $product_name = get_the_title($product_id);

            if($type === 'subscription') {

                $args = array(
                    'id' => 'plan_' . $product_id,
                    'currency' => strtolower(North_Checkout_Admin::get_current_currency()),
                    'interval' => $data['product_frequency_sub'],
                    'product' => ['name' => wp_strip_all_tags($product_name)],
                );

                if ($need_bump) {

                    $args_bump = array(
                        'id' => 'plan_bump_' . $product_id,
                        'currency' => strtolower(North_Checkout_Admin::get_current_currency()),
                        'interval' => isset($data['bump_frequency_sub']) ? $data['bump_frequency_sub'] : '',
                        'product' => ['name' => wp_strip_all_tags($product_name . ' (bump)')],
                    );

                }

                if (isset($data['product_start_price_sub']) && !empty($data['product_start_price_sub']) && floatval($data['product_start_price_sub']) > 0) {
                    $start_price = round(floatval($data['product_start_price_sub']) * 100, 2);
                    $args['billing_scheme'] = 'tiered';
                    $args['tiers_mode'] = 'graduated';
                    $args['tiers'] = [
                        [
                            "up_to" => 1,
                            "unit_amount" => $start_price,
                        ],
                        ["up_to" => "inf",
                            "unit_amount" => round(floatval($data['product_price']) * 100, 2),
                        ],
                    ];

                } else {
                    $args['amount'] = round(floatval($data['product_price']) * 100, 2);
                }

                if ($need_bump) {
                    if (isset($data['bump_start_price_sub']) && !empty($data['bump_start_price_sub'])) {
                        $bump_start_price = round(floatval($data['bump_start_price_sub']) * 100, 2);
                        $args_bump['billing_scheme'] = 'tiered';
                        $args_bump['tiers_mode'] = 'graduated';
                        $args_bump['tiers'] = [
                            [
                                "up_to" => 1,
                                "unit_amount" => $bump_start_price,
                            ],
                            ["up_to" => "inf",
                                "unit_amount" => isset($data['bump_price']) ? round(floatval($data['bump_price']) * 100, 2) : 0,
                            ],
                        ];
                    } else {
                        $args_bump['amount'] = isset($data['bump_price']) ? round(floatval($data['bump_price']) * 100, 2) : 0;
                    }
                }

            }else {

                $payment_number = intval($data['product_payment_number']) !== 0 ? intval($data['product_payment_number']) : 1;
                $bump_payment_number = intval($data['bump_payment_number']) !== 0 ? intval($data['bump_payment_number']) : 1;

                $payment_price = round((floatval($data['product_price'])) / $payment_number, 2) * 100;
                $bump_payment_price = round((floatval($data['bump_price'])) / $bump_payment_number, 2) * 100;

                $args = array(
                    'id' => 'plan_' . $product_id,
                    'currency' => strtolower(North_Checkout_Admin::get_current_currency()),
                    'interval' => $data['product_frequency'],
                    'product' => ['name' => wp_strip_all_tags($product_name)],
                );
                $args_bump = array();

                if($need_bump) {
                    $args_bump = array(
                        'id' => 'plan_bump_' . $product_id,
                        'currency' => strtolower(North_Checkout_Admin::get_current_currency()),
                        'interval' => $data['bump_frequency'],
                        'product' => ['name' => wp_strip_all_tags($product_name) . ' (bump)'],
                    );
                }

                if (isset($data['product_start_price']) && !empty($data['product_start_price']) && floatval($data['product_start_price']) > 0) {
                    $start_price = round(floatval($data['product_start_price']) * 100, 2);
                    $left_price = floatval($data['product_price']) - floatval($data['product_start_price']);
                    $args['billing_scheme'] = 'tiered';
                    $args['tiers_mode'] = 'graduated';
                    $args['tiers'] = [
                        [
                            "up_to" => 1,
                            "unit_amount" => $start_price,
                        ],
                        ["up_to" => "inf",
                            "unit_amount" => round(($left_price / $payment_number), 2) * 100,
                        ],
                    ];

                } else {
                    $args['amount'] = $payment_price;
                }

                if ($need_bump) {
                    if (isset($data['bump_start_price']) && !empty($data['bump_start_price'])) {
                        $bump_start_price = round(floatval($data['bump_start_price']) * 100, 2);
                        $bump_left_price = floatval($data['bump_price']) - floatval($data['bump_start_price']);
                        $args_bump['billing_scheme'] = 'tiered';
                        $args_bump['tiers_mode'] = 'graduated';
                        $args_bump['tiers'] = [
                            [
                                "up_to" => 1,
                                "unit_amount" => $bump_start_price,
                            ],
                            ["up_to" => "inf",
                                "unit_amount" => round(($bump_left_price / $bump_payment_number), 2) * 100,
                            ],
                        ];
                    } else {
                        $args_bump['amount'] = $bump_payment_price;
                    }
                }

            }

            if( !empty( get_option('stripe_private_key') ) ) {


                $response = $this->stripe = new \NorthStripe\StripeClient(get_option('stripe_private_key'));

                $response = $this->stripe->plans->create($args);


                if($need_bump) {

                    $response = $this->stripe->plans->create($args_bump);
                }
            }

            if( !empty( get_option('stripe_test_private_key') ) ) {

                $this->stripe = new \NorthStripe\StripeClient(get_option('stripe_test_private_key'));

                $response = $this->stripe->plans->create($args);

                if($need_bump) {

                    $response = $this->stripe->plans->create($args_bump);
                }
            }

        }
        catch (Stripe_Error $e) {
            do_action('north_failed_stripe_plan_created', $product_id, $data, $need_bump);
            return false;
        }
        catch (Exception $e) {
            do_action('north_failed_stripe_plan_created', $product_id, $data, $need_bump);
            return false;
        }

        do_action('north_after_stripe_plan_created', $product_id, $data, $need_bump);

        return true;
    }

    /**
     * Delete payment plans in stripe.
     *
     * @param $plan_id
     * @throws \NorthStripe\Exception\ApiErrorException
     * @since   1.0
     */
    function delete_plan($plan_id) {

        do_action('north_before_stripe_plan_deleted', $plan_id);

        if (!empty(get_option('stripe_private_key'))) {
            $this->stripe = new \NorthStripe\StripeClient(get_option('stripe_private_key'));

            try {
                $plan = $this->stripe->plans->retrieve(
                    'plan_' . $plan_id,
                    []
                );

                $this->stripe->plans->delete(
                    'plan_' . $plan_id,
                    []
                );

                $this->stripe->products->delete(
                    $plan->product,
                    []
                );
            } catch (Stripe_Error $e) {} catch (Exception $e) {}

            try {
                $plan = $this->stripe->plans->retrieve(
                    'plan_bump_' . $plan_id,
                    []
                );

                $this->stripe->plans->delete(
                    'plan_bump_' . $plan_id,
                    []
                );

                $this->stripe->products->delete(
                    $plan->product,
                    []
                );
            } catch (Stripe_Error $e) {} catch (Exception $e) {}
        }

        if (!empty(get_option('stripe_test_private_key'))) {
            $this->stripe = new \NorthStripe\StripeClient(get_option('stripe_test_private_key'));

            try {
                $plan = $this->stripe->plans->retrieve(
                    'plan_' . $plan_id,
                    []
                );

                $this->stripe->plans->delete(
                    'plan_' . $plan_id,
                    []
                );

                $this->stripe->products->delete(
                    $plan->product,
                    []
                );
            } catch (Stripe_Error $e) {} catch (Exception $e) {}

            try {
                $plan = $this->stripe->plans->retrieve(
                    'plan_bump_' . $plan_id,
                    []
                );

                $this->stripe->plans->delete(
                    'plan_bump_' . $plan_id,
                    []
                );

                $this->stripe->products->delete(
                    $plan->product,
                    []
                );
            } catch (Stripe_Error $e) {} catch (Exception $e) {}
        }

        do_action('north_after_stripe_plan_deleted', $plan_id);
    }

    function recreate_plans() {
        $args = array(
            'numberposts' => -1,
            'post_type'   => 'north_product',
            'post_status' => 'any',
            'meta_query' => array(
                'product_type' => array(
                    'key'     => 'product_type',
                    'value'   => array('Subscription', 'Payment Plan'),
                    'compare' => 'IN',
                ),
            ),
        );

        $products = get_posts( $args );

        if($this->is_connected()) {

            foreach ($products as $product) {

                $data = get_post_meta($product->ID);
                $new_data = array();

                foreach ($data as $key => $value) {
                    $new_data[$key] = $value[0];
                }

                if (isset($data['reload_bump']) && $data['reload_bump']) {
                    $need_bump = true;
                }

                $this->delete_plan($product->ID);
                $this->create_plan($product->ID, strtolower(str_replace(" ", "_", $new_data["product_type"])), $new_data, $need_bump);

                update_post_meta($product->ID, 'stripe_active', 'active');
            }
        }
    }

    /**
     * Get state value for Stripe.
     *
     * @return string state value
     */
    function get_state() {
        return get_option( 'stripe_token');
    }

    /**
     * Compare saved token and Stripe token.
     *
     * @param $stateParameter
     * @return bool match result
     */
    function state_matches($stateParameter) {
        $savedState = get_option('stripe_token');

        return $savedState == $stateParameter;
    }

    /**
     * Connect Stripe, check for correct api key and save keys to database.
     *
     * @param $code
     */
    function connect($code) {

        $product = new North_Checkout_Product();

        if( get_option('stripe_unique') && $code === get_option('stripe_unique') ){
            return;
        }

        try {
            \NorthStripe\Stripe::setApiKey('sk_live_hlL3TWO6vS1YvzieS9JbBXU800MsWIY7Ni');

            $response_live = \NorthStripe\OAuth::token([
                'grant_type' => 'authorization_code',
                'code' => $code,
            ]);

            \NorthStripe\Stripe::setApiKey('sk_test_chpSo9eXqSEsO9Q8dSc8K6tC006QYoVZsj');

            $response_test = \NorthStripe\OAuth::token([
                'grant_type' => 'refresh_token',
                'refresh_token' => $response_live->refresh_token,
            ]);

            update_option('stripe_public_key', $response_live->stripe_publishable_key);
            update_option('stripe_private_key', $response_live->access_token);
            update_option('stripe_test_public_key', $response_test->stripe_publishable_key);
            update_option('stripe_test_private_key',  $response_test->access_token);

            if(North_Checkout_Admin::is_plugin_activated()) {
                update_option('stripe_public_active', $response_live->stripe_publishable_key);
                update_option('stripe_private_active', $response_live->access_token);
            }else {
                update_option('stripe_public_active', $response_test->stripe_publishable_key);
                update_option('stripe_private_active', $response_test->access_token);
                update_option('test_stripe', 'on');
            }

            update_option('stripe_user_id', $response_live->stripe_user_id);
            update_option('stripe_unique', $code);

            $product->sync_products($stripe_sync = true, $paypal_products = false);
        }
        catch (Stripe_Error $e) {
            echo '<div class="north-error" style="display: block;">' . esc_html( $e ) . '</div>';
        }
        catch (Exception $e) {
            echo '<div class="north-error" style="display: block;">' . esc_html( $e ) . '</div>';
        }

        try {
            \NorthStripe\Stripe::setApiKey(get_option('stripe_private_key'));

            \NorthStripe\ApplePayDomain::create([
                'domain_name' => isset($_SERVER['SERVER_NAME']) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_NAME'] ) ) : '',
            ]);
        }
        catch (Stripe_Error $e) {
            echo '<div class="north-error" style="display: block;">You won\'t be able to use apple pay until you add your apple developer merchant-id. Follow <a href="https://stripe.com/docs/stripe-js/elements/payment-request-button#verifying-your-domain-with-apple-pay" target="_blank">this tutorial</a></div>';
        }
        catch (Exception $e) {
            echo '<div class="north-error" style="display: block;">You won\'t be able to use apple pay until you add your apple developer merchant-id. Follow <a href="https://stripe.com/docs/stripe-js/elements/payment-request-button#verifying-your-domain-with-apple-pay" target="_blank">this tutorial</a></div>';
        }
    }

    /**
     * Connect Stripe, check for correct api key and save keys to database.
     *
     */
    function disconnect() {

        if( !get_option('stripe_unique')){
            return;
        }

        delete_option('stripe_public_key');
        delete_option('stripe_private_key');
        delete_option('stripe_test_private_key');
        delete_option('stripe_test_private_key');
        delete_option('stripe_public_active');
        delete_option('stripe_private_active');
        delete_option('stripe_user_id');
        delete_option('test_stripe');

        wp_die();
    }

    /**
     * Check stripe connection.
     *
     * @return bool
     */
    function is_connected() {

        if(!empty(get_option('stripe_private_active'))) {
            return get_option('stripe_private_active');
        }

        return false;
    }

    /**
     * Get stripe account name.
     *
     * @param $key
     * @param $payment_system
     * @return Account
     */
    function get_account_name($key) {

        try {
            \NorthStripe\Stripe::setApiKey($key);

            $akk =  \NorthStripe\Account::retrieve();

            return $akk->settings->dashboard->display_name;
        }
        catch (Stripe_Error $e) {
            echo '<div class="north-error" style="display: block;">' . esc_html( $e ) . '</div>';

            $headers = array('content-type: text/html');
            $message = 'Something wrong with your Stripe account. Check Your <a href="'. site_url() .'/wp-admin/admin.php?page=north-checkout-settings">Settings Tab</a>, please.';
            wp_mail(get_option('admin_email'), 'North Checkout - Stripe Error', $message, $headers);
        }
        catch (Exception $e) {
            echo '<div class="north-error" style="display: block;">' . esc_html( $e ) . '</div>';

            $headers = array('content-type: text/html');
            $message = 'Something wrong with your Stripe account. Check Your <a href="'. site_url() .'/wp-admin/admin.php?page=north-checkout-settings">Settings Tab</a>, please.';
            wp_mail(get_option('admin_email'), 'North Checkout - Stripe Error', $message, $headers);
        }
    }
}
