<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://northplugins.com/
 * @since       1.0
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since       1.0
 * @package    North_Checkout
 * @subpackage North_Checkout/includes
 * @author     North Plugins <hello@northcheckout.com>
 */
class North_Checkout {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0
	 * @access   protected
	 * @var      North_Checkout_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0
	 */
	public function __construct() {
		if ( defined( 'NORTH_CHECKOUT_VERSION' ) ) {
			$this->version = NORTH_CHECKOUT_VERSION;
		} else {
			$this->version = '1.0';
		}
		$this->plugin_name = 'north-checkout';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - North_Checkout_Loader. Orchestrates the hooks of the plugin.
	 * - North_Checkout_i18n. Defines internationalization functionality.
	 * - North_Checkout_Admin. Defines all hooks for the admin area.
	 * - North_Checkout_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-north-checkout-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-north-checkout-i18n.php';

        /**
         * The class use for clean form data.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-clean.php';

        /**
         * The class responsible for defining paypal.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-paypal.php';

        /**
         * The class responsible for defining stripe.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-stripe.php';

        /**
         * The class responsible for defining order.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-order.php';

        /**
         * The class responsible for defining product.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-product.php';

        /**
         * The class responsible for defining coupon.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-coupon.php';

        /**
         * The class responsible for adding deactivating popup feedback.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-deactivate-popup.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-north-checkout-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-north-checkout-public.php';

        /**
         * Including stripe
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'libs/Stripe/init.php';

		$this->loader = new North_Checkout_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the North_Checkout_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new North_Checkout_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new North_Checkout_Admin( $this->get_plugin_name(), $this->get_version() );
		$paypal = new North_Checkout_Paypal();
		$stripe = new North_Checkout_Stripe();
        $order = new North_Checkout_Order();
        $product = new North_Checkout_Product();
        $coupon = new North_Checkout_Coupon();

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menu' );
		$this->loader->add_action( 'wp_ajax_switch_payment_mode', $plugin_admin, 'switch_payment_mode' );
		$this->loader->add_action( 'wp_ajax_stripe_disconnect', $stripe, 'disconnect' );
		$this->loader->add_action( 'wp_ajax_paypal_disconnect', $paypal, 'disconnect' );
        $this->loader->add_filter( 'plugins_api', $plugin_admin, 'north_plugin_info', 20, 3);
        $this->loader->add_filter( 'site_transient_update_plugins', $plugin_admin, 'north_push_update' );
        $this->loader->add_action( 'upgrader_process_complete', $plugin_admin,'north_after_update', 10, 2 );
		$this->loader->add_action( 'init', $order, 'register_order');
        $this->loader->add_action( 'wp_ajax_delete_order', $order, 'delete_order' );
		$this->loader->add_action( 'init', $product, 'register_product');
		$this->loader->add_action( 'wp_ajax_get_orders', $order, 'get_orders' );
        $this->loader->add_action( 'rest_api_init', $order, 'register_order_route');
        $this->loader->add_action( 'rest_api_init', $product, 'register_product_route');
        $this->loader->add_action( 'wp_ajax_get_products', $product, 'get_products' );
        $this->loader->add_action( 'wp_ajax_delete_product', $product, 'delete_product' );
        $this->loader->add_action( 'wp_ajax_sync_products', $product, 'sync_products' );
        $this->loader->add_action( 'wp_ajax_delete_coupon', $coupon, 'delete_coupon' );
        $this->loader->add_action( 'wp_ajax_get_coupons', $coupon, 'get_coupons' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new North_Checkout_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
        $this->loader->add_action( 'divi_extensions_init', $plugin_public, 'north_initialize_extension' );
        $this->loader->add_action( 'wp_footer', $plugin_public, 'insert_modal_thank' );
        $this->loader->add_action( 'wp_ajax_stripe_payment_process', $plugin_public, 'stripe_payment_process' );
        $this->loader->add_action( 'wp_ajax_nopriv_stripe_payment_process', $plugin_public, 'stripe_payment_process' );
        $this->loader->add_action( 'wp_ajax_paypal_payment_process', $plugin_public, 'paypal_payment_process' );
        $this->loader->add_action( 'wp_ajax_nopriv_paypal_payment_process', $plugin_public, 'paypal_payment_process' );
		$this->loader->add_action( 'wp_ajax_payment_buttons', $plugin_public, 'north_payment_buttons' );
		$this->loader->add_action( 'wp_ajax_nopriv_payment_buttons', $plugin_public, 'north_payment_buttons' );
        $this->loader->add_action( 'wp_ajax_paypal_order_create', $plugin_public, 'paypal_order_create' );
        $this->loader->add_action( 'wp_ajax_nopriv_paypal_order_create', $plugin_public, 'paypal_order_create' );
        $this->loader->add_action( 'wp_ajax_payment_upsell', $plugin_public, 'north_payment_upsell' );
        $this->loader->add_action( 'wp_ajax_nopriv_payment_upsell', $plugin_public, 'north_payment_upsell' );
        $this->loader->add_action( 'wp_ajax_apply_coupon', $plugin_public, 'north_apply_coupon' );
        $this->loader->add_action( 'wp_ajax_nopriv_apply_coupon', $plugin_public, 'north_apply_coupon' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since      1.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since      1.0
	 * @return    North_Checkout_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since      1.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
