<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://northplugins.com/
 * @since       1.0
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    North_Checkout
 * @subpackage North_Checkout/public
 * @author     North Plugins <hello@northcheckout.com>
 */
class North_Checkout_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * The stripe private key.
     *
     * @since    1.0
     * @access   private
     * @var      string    $privat_key    The current stripe private key.
     */
    private $private_key;

    /**
     * @var     NorthStripe\StripeClient  $stripe  Stripe client instance.
     * @access  private
     */
    private $stripe;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->private_key = get_option('stripe_private_active');

        if ( ! empty( $this->private_key ) ) {
            $this->stripe = new \NorthStripe\StripeClient( $this->private_key );
        }

    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0
     */
    public function enqueue_styles() {

        wp_enqueue_style('font-awesome', plugin_dir_url( __FILE__ ) . 'src/fontawesome/css/all.css', array(), $this->version, 'fas');
        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/north-checkout-public.css', array(), $this->version, 'all' );
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0
     */
    public function enqueue_scripts() {

        $paypal = new North_Checkout_Paypal();
        $current_currency = !empty(get_option('current_currency')) ? get_option('current_currency') : 'USD';

        wp_enqueue_script('stripe-js', 'https://js.stripe.com/v3/', array(), '');
        wp_enqueue_script( $this->plugin_name . '-stripe', plugin_dir_url( __FILE__ ) . 'js/north-checkout-stripe.js', array( 'jquery' ), $this->version, true );

        if( !empty(get_option('test_paypal')) ){
            if(!empty($paypal->is_connected('test'))) {
                wp_enqueue_script( $this->plugin_name . '-paypal-sdk', 'https://www.paypal.com/sdk/js?client-id='.$paypal->get_client_test().'&vault=true&disable-funding=credit,card&currency=' . $current_currency, null, null, true);
            }
        }else {
            if (!empty($paypal->is_connected('live'))) {
                wp_enqueue_script($this->plugin_name . '-paypal-sdk', 'https://www.paypal.com/sdk/js?client-id=' . $paypal->get_client_live() . '&vault=true&disable-funding=credit,card&currency=' . $current_currency, null, null, true);
            }
        }

        wp_enqueue_script( $this->plugin_name . '-paypal', plugin_dir_url( __FILE__ ) . 'js/north-checkout-paypal.js', array( 'jquery' ), $this->version, true );
        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/north-checkout-public.js', array( 'jquery' ), $this->version, true );

        if( !empty(get_option('north_google_key')) ) {
            wp_enqueue_script('google-places', 'https://maps.googleapis.com/maps/api/js?key=' . get_option('north_google_key') . '&libraries=places&callback=initAutocomplete', array(), null, true);
        }

        wp_localize_script( $this->plugin_name, 'ajax_object',
            array(
                'ajaxurl' => admin_url( 'admin-ajax.php' ),
                'order_paged' => 2,
                'product_paged' => 2
            )
        );
        wp_localize_script( $this->plugin_name . '-stripe', 'stripe_data', [
            'public' => get_option( 'stripe_public_active' ),
        ] );
    }

    public function insert_modal_thank() {

        echo '<div class="modal modal-checkout">
			     <div class="modal-inner">
					<h2>Thanks for your Order!</h2>
					<div class="modal-errors"></div>
					<img src="' . esc_url( plugin_dir_url( __FILE__) ) . 'images/thank.png" alt="img">
					<a class="modal-close">OK</a>
                 </div>
              </div>';

    }

    public function north_initialize_extension() {

        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-north-checkout-module.php';

    }

    /**
     * Initialize the payment for product.
     *
     * @param object $product Product object
     * @param integer $customer_id
     * @param array $variations Product variations
     * @param null $payment_id
     * @return array|Exception|Stripe_Error
     * @since    1.0
     */
    public function product_payment($product, $additional_sum, $coupon, $customer_id, $quantity = 1, $is_bump_payment = false, $payment_id = null, $variations = array()) {

        do_action('north_before_stripe_payment', $product, $customer_id, $quantity, $is_bump_payment);

        $transaction = null;
        $subtotal = null;
        $currency = null;
        $quantity = $quantity !== null ? $quantity : 1;
        $coupon = get_page_by_title($coupon, OBJECT, 'north_coupon');
        $coupon_price = 0;
        $coupon_type = null;
        $coupon_id = null;
        $total = null;

        if(!empty($coupon)) {
            $coupon_id = $coupon->ID;
            $coupon_status = $coupon->post_status;
            $coupon_instance = new North_Checkout_Coupon($coupon_id);
            $coupon_limit = get_post_meta($coupon_id, 'coupon_limit', true);

            if($coupon_status == 'active' && (empty($coupon_limit) || intval($coupon_limit) > 0)) {
                $coupon_type = get_post_meta($coupon_id, 'coupon_type', true);
                $coupon_price = floatval(get_post_meta($coupon_id, 'coupon_price', true));
            }
        }

        if(is_array($product)) {
            $product_price = floatval($product['dynamic_price']);
            $subtotal = floatval($product['dynamic_price']) + floatval($additional_sum);
            $total = floatval($product['dynamic_price']) + floatval($additional_sum);

            if($coupon_type != null) {
                if($coupon_type == 'fixed') {
                    $total = $subtotal - $coupon_price;
                }else {
                    $percent = $subtotal / 100 * $coupon_price;
                    $total = $subtotal - $percent;
                }

                $total = $total < 0 ? 0 : $total;
            }

            if ( $total > 0 ) {
                $currency = North_Checkout_Admin::get_current_currency();
                try {
                    if ( isset( $payment_id ) ) {
                        $transaction = $this->stripe->paymentIntents->create( [
                            'amount' => $total * 100,
                            'currency' => strtolower( $currency ),
                            'customer' => $customer_id,
                            'payment_method' => $payment_id,
                        ] );

                        $this->stripe->paymentIntents->confirm(
                            $transaction->id,
                            [ 'payment_method' => $payment_id ]
                        );
                    } else {
                        $transaction = $this->stripe->charges->create( [
                            'amount' => $total * 100,
                            'currency' => strtolower( $currency ),
                            'customer' => $customer_id,
                        ] );
                    }
                } catch ( Stripe_Error $e ) {
                    return array('success' => false, 'error' => $e, 'transaction_id' => $transaction->id, 'single_price' => $product_price, 'subtotal' => $product_price * $quantity, 'total' => $total, 'coupon' => $coupon);
                } catch ( Exception $e ) {
                    return array('success' => false, 'error' => $e, 'transaction_id' => $transaction->id, 'single_price' => $product_price, 'subtotal' => $product_price * $quantity, 'total' => $total, 'coupon' => $coupon);
                }
            } else {
                do_action('north_after_stripe_payment', $product, $customer_id, $quantity, $is_bump_payment);
                return array('success' => true, 'transaction_id' => '', 'single_price' => $product_price, 'subtotal' => $subtotal, 'total' => $total, 'coupon' => $coupon_id);
            }


        }else {
            $product_id = $product->get_id();

            if( $is_bump_payment && $product->reload_bump() ) {
                $product_price = $product->get_bump_price();
                $subtotal = $product->get_bump_price();
                $total = $product->get_bump_price();
                $currency = North_Checkout_Admin::get_current_currency();
                $trial = $product->get_bump_trial_count();
                $trial_type = $product->get_bump_trial_type();
                $plan_prefix = 'plan_bump_';
            }else {
                $product_price = $product->get_price();
                $subtotal = $product->get_price();
                $total = $product->get_price();
                $currency = North_Checkout_Admin::get_current_currency();
                $trial = $product->get_trial_count();
                $trial_type = $product->get_trial_type();
                $plan_prefix = 'plan_';
            }

            try {
                if ( 'Subscription' === $product->get_type() ) {

                    if( $is_bump_payment && $product->reload_bump() ) {
                        if ($start_product_price = $product->get_bump_start_price()) {
                            $subtotal = $start_product_price;
                            $product_price = $start_product_price;
                            $total = $subtotal;
                        }
                    }else {
                        if ($start_product_price = $product->get_start_price()) {
                            $subtotal = $start_product_price;
                            $product_price = $start_product_price;
                            $total = $subtotal;
                        }
                    }

                    $args = array(
                        'customer' => $customer_id,
                        'items' => [ [ 'plan' => $plan_prefix . $product->get_id() ] ],
                    );

                    if ( isset( $payment_id ) ) {
                        $args['default_payment_method'] = $payment_id;
                    }

                    if ( $trial != null || $trial != 0 ) {
                        $args['trial_end'] = strtotime( $trial . ' ' . $trial_type );
                    }

                    $transaction = $this->stripe->subscriptions->create( $args );

                    if ( $additional_sum != 0 ) {
                        $total += $additional_sum;
                        if ( isset( $payment_id ) ) {
                            $transaction = $this->stripe->paymentIntents->create([
                                'amount' => $additional_sum * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                                'payment_method' => $payment_id,
                            ]);

                            $this->stripe->paymentIntents->confirm(
                                $transaction->id,
                                ['payment_method' => $payment_id]
                            );
                        } else {
                            $transaction = $this->stripe->charges->create([
                                'amount' => $additional_sum * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                            ]);
                        }
                    }

                } elseif ( 'Payment Plan' === $product->get_type() ) {

                    if ( $is_bump_payment && $product->reload_bump() ) {
                        $frequency = $product->get_bump_frequency();
                        $payment_count = $product->get_bump_payment_count();

                        if ($start_product_price = $product->get_bump_start_price()) {
                            $product_price = $start_product_price + floatval($additional_sum);
                            $subtotal = $start_product_price + floatval($additional_sum);
                            $total = $subtotal;
                        } else {
                            $product_price = round((floatval($subtotal) / $payment_count), 2);
                            $subtotal = round((floatval($subtotal) / $payment_count), 2);
                            $total = $subtotal;
                        }
                    } else {
                        $frequency = $product->get_frequency();
                        $payment_count = $product->get_payment_count();

                        if ($start_product_price = $product->get_start_price()) {
                            $product_price = $start_product_price;
                            $subtotal = $start_product_price;
                            $total = $subtotal;
                        } else {
                            $product_price = round((floatval($subtotal) / $payment_count), 2);
                            $subtotal = round((floatval($subtotal) / $payment_count), 2);
                            $total = $subtotal;
                        }
                    }

                    $cancel_time = strtotime(strval($payment_count) . " " . $frequency);

                    $transaction_args = [
                        'customer' => $customer_id,
                        'items' => [ [ 'plan' => $plan_prefix . $product->get_id() ] ],
                        'cancel_at' => $cancel_time,
                    ];
                    if ( isset( $payment_id ) ) {
                        $transaction_args[ 'default_payment_method' ] = $payment_id;
                    }
                    $transaction = $this->stripe->subscriptions->create( $transaction_args );

                    if ( $additional_sum != 0 ) {
                        $total += $additional_sum;
                        if ( isset( $payment_id ) ) {
                            $transaction = $this->stripe->paymentIntents->create([
                                'amount' => $additional_sum * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                                'payment_method' => $payment_id,
                            ]);

                            $this->stripe->paymentIntents->confirm(
                                $transaction->id,
                                ['payment_method' => $payment_id]
                            );
                        } else {
                            $transaction = $this->stripe->charges->create([
                                'amount' => $additional_sum * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                            ]);
                        }
                    }

                } else {

                    if ( $variations ) {
                        $product_price = $product->get_variation_price( $product_id, $variations );
                    }

                    $subtotal = $product_price * $quantity + $additional_sum;
                    $total = $subtotal;

                    if ( isset( $coupon_type ) ) {
                        if ( 'fixed' === $coupon_type ) {
                            $total = $subtotal - $coupon_price;
                        } else {
                            $percent = $subtotal / 100 * $coupon_price;
                            $total = $subtotal - $percent;
                        }
                        $total = $total < 0 ? 0 : $total;
                    }

                    if ( $total > 0 ) {

                        if ( isset( $payment_id ) ) {
                            $transaction = $this->stripe->paymentIntents->create([
                                'amount' => $total * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                                'payment_method' => $payment_id,
                            ]);

                            $this->stripe->paymentIntents->confirm(
                                $transaction->id,
                                ['payment_method' => $payment_id]
                            );
                        } else {
                            $transaction = $this->stripe->charges->create([
                                'amount' => $total * 100,
                                'currency' => strtolower($currency),
                                'customer' => $customer_id,
                            ]);
                        }

                    } else {

                        do_action('north_after_stripe_payment', $product, $customer_id, $quantity, $is_bump_payment);

                        return array('success' => true, 'transaction_id' => '', 'single_price' => $product_price, 'subtotal' => $subtotal, 'total' => $total, 'coupon' => $coupon_id);

                    }
                }
            } catch ( Stripe_Error $e ) {
                do_action('north_failed_stripe_payment', $product, $e, $transaction->id);
                return array('success' => false, 'error' => $e, 'transaction_id' => $transaction->id, 'single_price' => $product_price, 'subtotal' => $subtotal, 'total' => $total, 'coupon' => $coupon_id);
            } catch ( Exception $e ) {
                do_action('north_failed_stripe_payment', $product, $e, $transaction->id);
                return array('success' => false, 'error' => $e, 'transaction_id' => $transaction->id, 'single_price' => $product_price, 'subtotal' => $subtotal, 'total' => $total, 'coupon' => $coupon_id);
            }
        }

        do_action('north_after_stripe_payment', $product, $customer_id, $quantity, $is_bump_payment);

        return array('success' => true, 'transaction_id' => $transaction->id, 'single_price' => $product_price, 'subtotal' => $subtotal, 'total' => $total, 'coupon' => $coupon_id);

    }

    /**
     * Payment process paypal.
     *
     */
    public function paypal_payment_process() {

        parse_str(isset( $_POST['form'] ) ? $_POST['form'] : '', $form_data); //phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized, WordPress.Security.NonceVerification.Missing

        do_action('north_before_paypal_payment', $form_data);

        $product_id = $form_data['checkout_product'];
        $order_data = array();
        $quantity   = 1;
        $current_module = $form_data['product_module'];
        $coupon = $form_data['checkout_discount_input'];

        $coupon = get_page_by_title($coupon, OBJECT, 'north_coupon');
        $coupon_price = 0;
        $coupon_type = null;

        if(!empty($coupon)) {
            $coupon_id = $coupon->ID;
            $coupon_status = $coupon->post_status;
            $coupon_instance = new North_Checkout_Coupon($coupon_id);
            $coupon_limit = intval(get_post_meta($coupon_id, 'coupon_limit', true));

            if($coupon_status == 'active' && (empty($coupon_limit) || intval($coupon_limit) > 0)) {

                $coupon_type = get_post_meta($coupon_id, 'coupon_type', true);
                $coupon_price = floatval(get_post_meta($coupon_id, 'coupon_price', true));
            }
        }

        if(isset($form_data['quantity_' . $current_module])) {
            $order_data['product_quantity'] = $form_data['quantity_' . $current_module];
            $quantity = intval($order_data['product_quantity']);
        }
        else {
            $order_data['product_quantity'] = $quantity;
        }

        if(strpos($product_id, 'dynamic_') !== false) {

            $module_num = str_replace('dynamic_', '', $product_id);
            $order_data['product_currency'] = $form_data['dynamic_currency_' . $module_num];

            $total_price = floatval($form_data['dynamic_price_' . $module_num]);

            if($coupon_type != null) {
                if($coupon_type == 'fixed') {
                    $total_price = $total_price - $coupon_price;
                }else {
                    $percent = $total_price / 100 * $coupon_price;
                    $total_price = $total_price - $percent;
                }

                $total_price = $total_price < 0 ? 0 : $total_price;
            }

            $order_data['product_price'] = $total_price;

        }else {
            $product = new North_Checkout_Product($product_id);
            $total = 0;
            $order_data['product_currency'] = strtoupper(North_Checkout_Admin::get_current_currency());
            $prefix = 'live';

            if (!empty(get_option('test_paypal'))) {
                $prefix = 'test';
            }

            if ($product->get_type() == "Subscription" || $product->get_type() == "Payment Plan") {
                do_action('north_after_paypal_payment', $form_data);
                wp_die(json_encode(array('id' => get_post_meta($product_id, 'paypal_' . $prefix . '_product_id', true))));
            } else {
                if ($product->is_variable()) {

                    $variations = array();

                    foreach ($form_data as $key => $value) {

                        if (!empty($value)) {
                            if (strpos($key, 'variation_') !== false) {
                                if ($current_module === substr($key,strrpos($key, '_')+1)) {
                                    $variable_key = substr($key, 10);
                                    $variable_key = substr($variable_key, 0, strrpos($variable_key, "_"));
                                    $variations[$variable_key] = $value;
                                }
                            }
                        }
                    }
                    $total_price = $product->get_variation_price($product_id, $variations) * $quantity;

                    if($coupon_type != null) {
                        if($coupon_type == 'fixed') {
                            $total_price = $total_price - $coupon_price;
                        }else {
                            $percent = $total_price / 100 * $coupon_price;
                            $total_price = $total_price - $percent;
                        }

                        $total_price = $total_price < 0 ? 0 : $total_price;
                    }

                    $order_data['product_price'] = $total_price;

                } else {
                    $total_price = $product->get_price() * $quantity;

                    if($coupon_type != null) {
                        if($coupon_type == 'fixed') {
                            $total_price = $total_price - $coupon_price;
                        }else {
                            $percent = $total_price / 100 * $coupon_price;
                            $total_price = $total_price - $percent;
                        }

                        $total_price = $total_price < 0 ? 0 : $total_price;
                    }

                    $order_data['product_price'] = $total_price;

                }
            }
        }

        do_action('north_after_paypal_payment', $form_data);

        wp_die(json_encode($order_data));
    }

    public function stripe_payment_process() {

        parse_str(isset( $_POST['form'] ) ? $_POST['form'] : '', $form_data); //phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized, WordPress.Security.NonceVerification.Missing

        $order                  = new North_Checkout_Order();
        $product_id             = $form_data['checkout_product'];
        $bump_id                = $form_data['checkout_bump'];
        $card_token             = $form_data['stripe_token'];
        $flex_info              = array();
        $customer_address       = array();
        $variations             = array();
        $order_data             = array();
        $args                   = array();
        $quantity               = 1;
        $subtotal               = 0;
        $total                  = 0;
        $additional_sum         = 0;
        $product_payment_status = 'true';
        $bump_payment_status    = 'true';
        $current_module         = $form_data['product_module'];
        $coupon                 = $form_data['checkout_discount_input'];

        if ( isset( $form_data[ 'quantity_' . $current_module ] ) ) {
            $order_data['product_quantity'] = $form_data[ 'quantity_' . $current_module ];
            $quantity = intval($order_data['product_quantity']);
        } else {
            $order_data['product_quantity'] = $quantity;
        }

        if(strpos($product_id, 'dynamic_') !== false) {
            $module_num = str_replace('dynamic_', '', $product_id);
            $dynamic_price = floatval($form_data['dynamic_price_' . $module_num]);
            $currency = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['product_currency'] = $currency;
            $order_data['product_price_html'] = '<b>Product: </b> Dynamic price: ' . North_Checkout_Admin::formatted_price($dynamic_price, true, $currency);
            $order_data['product_type'] = 'One time';
            $order_data['card_last_four'] = $form_data['card_last_four'];
            $order_data['card_brand'] = $form_data['card_brand'];

        } else {
            $product = new North_Checkout_Product($product_id);

            if( $product->get_price() != 0 && empty($card_token)) {
                echo "Error. Invalid card. Please fill card fields.";
                wp_die();
            }

            $order_data['product_price_html'] = $product->get_price_html();
            $order_data['product_currency'] = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['product_type'] = $product->get_type();
            $order_data['card_last_four'] = $form_data['card_last_four'];
            $order_data['card_brand'] = $form_data['card_brand'];
        }

        // Write customer info
        foreach($form_data as $key => $value) {

            if (!empty($value)) {
                if( strpos($key, 'checkout_text_') !== false ) {

                    $flex_info['checkout' . substr($key, 13)] = $value;

                }elseif( strpos($key, 'checkout_radio_') !== false ) {

                    $flex_info['checkout' . substr($key, 14)] = $value;

                }elseif( strpos($key, 'checkout_checkbox_') !== false ) {

                    $flex_info['checkout' . substr($key, 17)] = $value;

                }elseif( strpos($key, 'checkout_select_') !== false ) {

                    $flex_info['checkout' . substr($key, 15)] = $value;

                }elseif( strpos($key, 'checkout_email_') !== false ) {

                    $order_data['checkout_email'] = $value;

                }elseif( strpos($key, 'checkout_first_') !== false ) {

                    $order_data['checkout_first'] = $value;

                }elseif( strpos($key, 'checkout_last_') !== false ) {

                    $order_data['checkout_last'] = $value;

                }elseif( strpos($key, 'checkout_phone_') !== false ) {

                    $order_data['checkout_phone'] = $value;

                }elseif( strpos($key, 'variation_') !== false  && isset($product) && $product->is_variable() ) {

                    if ($current_module === substr($key,strrpos($key, '_')+1)) {
                        $variable_key = substr($key, 10);
                        $variable_key = substr($variable_key, 0, strrpos($variable_key, "_"));
                        $variations[$variable_key] = $value;
                    }
                }
            }
        }

        if (isset($product) && $product->is_variable()) {
            $order_data['variations'] = $variations;
        }

        if(isset($form_data['checkout_address'])) {
            $customer_address = array(
                'full' => $form_data['checkout_address'] ? $form_data['checkout_address'] : 'Address not found',
                'line_2' => $form_data['checkout_line_2'] ? $form_data['checkout_line_2'] : '',
                'country' => $form_data['checkout_country'] ? $form_data['checkout_country'] : '',
                'city' => $form_data['checkout_city'] ? $form_data['checkout_city'] : '',
                'state' => $form_data['checkout_state'] ? $form_data['checkout_state'] : '',
                'zip' => $form_data['checkout_zip'] ?  $form_data['checkout_zip'] : '',
            );
        }

        if( !empty($card_token) ) {
            try {

                $customer = $this->stripe->customers->create([
                    'address' => array(
                        'line1' => $customer_address['full'],
                        'line2' => $customer_address['line_2'],
                        'city' => $customer_address['city'],
                        'country' => $customer_address['country'],
                        'postal_code' => $customer_address['zip'],
                        'state' => $customer_address['state']
                    ),
                    'email' => $order_data['checkout_email'],
                    'phone' => $order_data['checkout_phone'],
                    'name' => $order_data['checkout_first'] . " " . $order_data['checkout_last'],
                    'source' => $card_token,
                ]);

                setcookie('customer_id', $customer->id, 0, "/");

            } catch ( Stripe_Error $e ) {
                echo esc_html( $e );
                wp_die();
            } catch ( Exception $e ) {
                echo esc_html( $e );
                wp_die();
            }
        }

        if (isset($bump_id) && !empty($bump_id)) {
            $bump = new North_Checkout_Product($bump_id);
            $bump_price = 0;
            if($bump->reload_bump()) {
                $bump_price = $bump->get_bump_price();
                $order_data['bump_price_html'] = $bump->get_bump_price_html();
            }else {
                $bump_price = $bump->get_price();
                $order_data['bump_price_html'] = $bump->get_price_html();
            }

            if( $bump_price != 0) {

                if($bump->get_type() != 'One time') {
                    $additional_sum = 0;
                    $bump_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity, true);
                    $order_data['bump_price'] = $bump_result['single_price'];
                    $order_data['bump_subtotal'] = $bump_result['subtotal'];
                    $order_data['bump_trans_id'] = $bump_result['transaction_id'];
                    $order_data['bump_id'] = $bump_id;
                }else {
                    $additional_sum = floatval($bump_price);
                    $order_data['bump_price'] = $bump_price;
                    $order_data['bump_subtotal'] = $bump_price;
                }
                $order_data['bump_id'] = $bump_id;

            }else {
                $order_data['bump_id'] = $bump_id;
                $order_data['bump_subtotal'] = 0;
                $total = 0;
            }

            $order_data['bump_currency'] = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['bump_type'] = $bump->get_type();

        }

        if(strpos($product_id, 'dynamic_') !== false) {
            $module_num = str_replace('dynamic_', '', $product_id);
            $dynamic_name = $form_data['dynamic_name_' . $module_num];

            if($dynamic_price !== 0.0) {

                $product_result = $this->product_payment(array("dynamic_price" => $dynamic_price, "dynamic_currency" => $form_data['dynamic_currency_' . $module_num]), $additional_sum, $coupon, $customer->id);

                $order_data['product_name'] = $dynamic_name;
                $order_data['product_price'] = $product_result['single_price'];
                $order_data['product_subtotal'] = $product_result['subtotal'];
                $order_data['product_trans_id'] = $product_result['transaction_id'];
                $subtotal += $product_result['subtotal'];
                $total += $product_result['total'];

                if ($product_result['success'] !== true) {
                    echo "Product not added: " . esc_html( $product_result['error']->getError()->message );
                    $product_payment_status = 'false';
                }
            }else {
                $order_data['product_name'] = $dynamic_name;
                $order_data['product_price'] = 0;
                $order_data['product_subtotal'] = 0;
            }

        }else {

            if( isset($product) && $product->get_price() != 0) {

                if ($product->is_variable()) {
                    $product_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity, false, null, $variations);
                } else {
                    $product_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity);
                }

                $order_data['product_id'] = $product_id;
                $order_data['product_price'] = $product_result['single_price'];
                $order_data['product_subtotal'] = $product_result['subtotal'];
                $order_data['product_trans_id'] = $product_result['transaction_id'];
                $subtotal += $product_result['subtotal'];
                $total += $product_result['total'];

                if ($product_result['success'] !== true) {
                    echo "Product not added: " . esc_html( $product_result['error']->getError()->message );
                    $product_payment_status = 'false';
                }

            }else {
                $order_data['product_id'] = $product_id;
                $order_data['product_subtotal'] = 0;
            }
        }

        $order_data['coupon'] = $coupon;
        $order_data['subtotal'] = $subtotal;
        $order_data['total'] = $total;

        $order_status = 'warning';

        if($product_payment_status === 'false' && $bump_payment_status === 'false') {
            $order_status = 'failed';
        }

        if($product_payment_status === 'true' && $bump_payment_status === 'true') {
            $order_status = 'complete';
        }

        $order_data['product_payment_status'] = $product_payment_status;
        $order_data['bump_payment_status'] = $bump_payment_status;

        if ( $order_data['total'] <= 0 ) {

            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status, 'Free');

        } else {

            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status);

        }

        if ( ! is_wp_error( $order_id ) ) {
            setcookie('order_id', $order_id, 0, "/");
        }

        wp_die();

    }

    public function north_payment_buttons() {

        parse_str(isset( $_POST['form'] ) ? $_POST['form'] : '', $form_data); //phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized, WordPress.Security.NonceVerification.Missing

        $address = isset($_POST['customer_shipping']) ? sanitize_text_field( $_POST['customer_shipping'] ) : ''; //phpcs:ignore WordPress.Security.NonceVerification.Missing
        $name = explode(' ', isset($_POST['customer_name']) ? sanitize_text_field( $_POST['customer_name'] ) : ''); //phpcs:ignore WordPress.Security.NonceVerification.Missing
        $payment_id         = isset( $_POST['payment_id'] ) ? sanitize_text_field( $_POST['payment_id'] ) : ''; //phpcs:ignore WordPress.Security.NonceVerification.Missing

        $order                  = new North_Checkout_Order();
        $product_id             = $form_data['checkout_product'];
        $bump_id                = $form_data['checkout_bump'];
        $card_token             = $form_data['stripe_token'];
        $flex_info              = array();
        $customer_address       = array();
        $variations             = array();
        $order_data             = array();
        $args                   = array();
        $quantity               = 1;
        $subtotal               = 0;
        $total                  = 0;
        $additional_sum         = 0;
        $product_payment_status = 'true';
        $bump_payment_status    = 'true';
        $current_module         = $form_data['product_module'];
        $coupon                 = $form_data['checkout_discount_input'];

        if(isset($form_data['quantity_' . $current_module])) {
            $order_data['product_quantity'] = $form_data['quantity_' . $current_module];
            $quantity = intval($order_data['product_quantity']);
        }
        else {
            $order_data['product_quantity'] = $quantity;
        }

        if(strpos($product_id, 'dynamic_') !== false) {
            $module_num = str_replace('dynamic_', '', $product_id);
            $dynamic_price = floatval($form_data['dynamic_price_' . $module_num]);
            $currency = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['product_currency'] = $currency;
            $order_data['product_price_html'] = '<b>Product: </b> Dynamic price: ' . North_Checkout_Admin::formatted_price($dynamic_price, true, $currency);
            $order_data['product_type'] = 'One time';
            $order_data['card_last_four'] = $form_data['card_last_four'];
            $order_data['card_brand'] = $form_data['card_brand'];

        }else {
            $product = new North_Checkout_Product($product_id);

            if( $product->get_price() != 0 && empty($card_token)) {
                echo "Error. Invalid card. Please fill card fields.";
                wp_die();
            }

            $order_data['product_price_html'] = $product->get_price_html();
            $order_data['product_currency'] = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['product_type'] = $product->get_type();
            $order_data['card_last_four'] = $form_data['card_last_four'];
            $order_data['card_brand'] = $form_data['card_brand'];
        }

        // Write customer info
        foreach($form_data as $key => $value) {

            if (!empty($value)) {
                if( strpos($key, 'checkout_text_') !== false ) {

                    $flex_info['checkout' . substr($key, 13)] = $value;

                }elseif( strpos($key, 'checkout_radio_') !== false ) {

                    $flex_info['checkout' . substr($key, 14)] = $value;

                }elseif( strpos($key, 'checkout_checkbox_') !== false ) {

                    $flex_info['checkout' . substr($key, 17)] = $value;

                }elseif( strpos($key, 'checkout_select_') !== false ) {

                    $flex_info['checkout' . substr($key, 15)] = $value;

                }elseif( strpos($key, 'checkout_email_') !== false ) {

                    $order_data['checkout_email'] = $value;

                }elseif( strpos($key, 'checkout_first_') !== false ) {

                    $order_data['checkout_first'] = $value;

                }elseif( strpos($key, 'checkout_last_') !== false ) {

                    $order_data['checkout_last'] = $value;

                }elseif( strpos($key, 'checkout_phone_') !== false ) {

                    $order_data['checkout_phone'] = $value;

                }elseif( strpos($key, 'variation_') !== false  && isset($product) && $product->is_variable() ) {

                    if ($current_module === substr($key,strrpos($key, '_')+1)) {
                        $variable_key = substr($key, 10);
                        $variable_key = substr($variable_key, 0, strrpos($variable_key, "_"));
                        $variations[$variable_key] = $value;
                    }
                }
            }
        }

        if (isset($product) && $product->is_variable()) {
            $order_data['variations'] = $variations;
        }

        if(isset($form_data['checkout_address'])) {
            $customer_address = array(
                'full' => $form_data['checkout_address'] ? $form_data['checkout_address'] : 'Address not found',
                'line_2' => $form_data['checkout_line_2'] ? $form_data['checkout_line_2'] : '',
                'country' => $form_data['checkout_country'] ? $form_data['checkout_country'] : '',
                'city' => $form_data['checkout_city'] ? $form_data['checkout_city'] : '',
                'state' => $form_data['checkout_state'] ? $form_data['checkout_state'] : '',
                'zip' => $form_data['checkout_zip'] ?  $form_data['checkout_zip'] : '',
            );
        }

        if( !empty($card_token) ) {
            try {

                $customer = $this->stripe->customers->create([
                    'address' => array(
                        'line1' => $customer_address['full'],
                        'line2' => $customer_address['line_2'],
                        'city' => $customer_address['city'],
                        'country' => $customer_address['country'],
                        'postal_code' => $customer_address['zip'],
                        'state' => $customer_address['state']
                    ),
                    'email' => $order_data['checkout_email'],
                    'phone' => $order_data['checkout_phone'],
                    'name' => $order_data['checkout_first'] . " " . $order_data['checkout_last'],
                    'source' => $card_token,
                ]);

                setcookie('customer_id', $customer->id, 0, "/");

            } catch (Stripe_Error $e) {
                echo esc_html( $e );
                wp_die();
            } catch (Exception $e) {
                echo esc_html( $e );
                wp_die();
            }
        }

        if (isset($bump_id) && !empty($bump_id)) {
            $bump = new North_Checkout_Product($bump_id);
            $bump_price = 0;
            if($bump->reload_bump()) {
                $bump_price = $bump->get_bump_price();
                $order_data['bump_price_html'] = $bump->get_bump_price_html();
            }else {
                $bump_price = $bump->get_price();
                $order_data['bump_price_html'] = $bump->get_price_html();
            }

            if( $bump_price != 0) {

                if($bump->get_type() != 'One time') {
                    $additional_sum = 0;
                    $bump_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity, true, $payment_id);
                    $order_data['bump_price'] = $bump_result['single_price'];
                    $order_data['bump_subtotal'] = $bump_result['subtotal'];
                    $order_data['bump_trans_id'] = $bump_result['transaction_id'];
                    $order_data['bump_id'] = $bump_id;
                }else {
                    $additional_sum = floatval($bump_price);
                    $order_data['bump_price'] = $bump_price;
                    $order_data['bump_subtotal'] = $bump_price;
                }
                $order_data['bump_id'] = $bump_id;

            }else {
                $order_data['bump_id'] = $bump_id;
                $order_data['bump_subtotal'] = 0;
                $total = 0;
            }

            $order_data['bump_currency'] = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
            $order_data['bump_type'] = $bump->get_type();

        }

        if(strpos($product_id, 'dynamic_') !== false) {
            $module_num = str_replace('dynamic_', '', $product_id);
            $dynamic_name = $form_data['dynamic_name_' . $module_num];

            if($dynamic_price !== 0.0) {

                $product_result = $this->product_payment(array("dynamic_price" => $dynamic_price, "dynamic_currency" => $form_data['dynamic_currency_' . $module_num]), $additional_sum, $coupon, $customer->id, 1, false, $payment_id);

                $order_data['product_name'] = $dynamic_name;
                $order_data['product_price'] = $product_result['single_price'];
                $order_data['product_subtotal'] = $product_result['subtotal'];
                $order_data['product_trans_id'] = $product_result['transaction_id'];
                $subtotal += $product_result['subtotal'];
                $total += $product_result['total'];

                if ($product_result['success'] !== true) {
                    echo "Product not added: " . esc_html( $product_result['error']->getError()->message );
                    $product_payment_status = 'false';
                }
            }else {
                $order_data['product_name'] = $dynamic_name;
                $order_data['product_price'] = 0;
                $order_data['product_subtotal'] = 0;
            }

        }else {

            if( isset($product) && $product->get_price() != 0) {

                if ($product->is_variable()) {
                    $product_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity, false, $payment_id, $variations);
                } else {
                    $product_result = $this->product_payment($product, $additional_sum, $coupon, $customer->id, $quantity, false, $payment_id);
                }

                $order_data['product_id'] = $product_id;
                $order_data['product_price'] = $product_result['single_price'];
                $order_data['product_subtotal'] = $product_result['subtotal'];
                $order_data['product_trans_id'] = $product_result['transaction_id'];
                $subtotal += $product_result['subtotal'];
                $total += $product_result['total'];

                if ($product_result['success'] !== true) {
                    echo "Product not added: " . esc_html( $product_result['error']->getError()->message );
                    $product_payment_status = 'false';
                }

            }else {
                $order_data['product_id'] = $product_id;
                $order_data['product_subtotal'] = 0;
            }
        }

        $order_data['coupon'] = $coupon;
        $order_data['subtotal'] = $subtotal;
        $order_data['total'] = $total;

        $order_status = 'warning';

        if($product_payment_status === 'false' && $bump_payment_status === 'false') {
            $order_status = 'failed';
        }

        if($product_payment_status === 'true' && $bump_payment_status === 'true') {
            $order_status = 'complete';
        }

        $order_data['product_payment_status'] = $product_payment_status;
        $order_data['bump_payment_status'] = $bump_payment_status;


        if ( $order_data['total'] <= 0 ) {

            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status, 'Free');

        } else {

            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status);

        }

        if ( ! is_wp_error( $order_id ) ) {
            setcookie('order_id', $order_id, 0, "/");
        }

        wp_die();

    }

    public function north_payment_upsell() {

        do_action('north_before_payment_upsell');

        $product_id = isset( $_POST['product'] ) ? sanitize_text_field( $_POST['product'] ) : ''; //phpcs:ignore WordPress.Security.NonceVerification.Missing
        $order_id = isset( $_COOKIE['order_id'] ) ? sanitize_text_field( $_COOKIE['order_id'] ) : '';
        $product = new North_Checkout_Product($product_id);
        $customer_id = isset( $_COOKIE['customer_id'] ) ? sanitize_text_field( $_COOKIE['customer_id'] ) : '';
        $meta = get_post_meta($order_id);
        $orders_count = wp_count_posts( 'north_order' );
        $order_number = intval($orders_count->pending) + intval($orders_count->complete) + intval($orders_count->failed) + 1;
        $additional_sum = 0;

        $payment_result = $this->product_payment($product, $additional_sum, $customer_id);

        $order_args = array(
            'post_type'     => 'north_order',
            'post_title'    => wp_strip_all_tags( '#' . $order_number ),
            'post_status'   => 'complete',
        );

        $new_order = wp_insert_post( $order_args, true );

        update_post_meta($new_order, 'flex_info', $meta["flex_info"][0]);
        update_post_meta($new_order, 'customer_name', $meta["customer_name"][0]);
        update_post_meta($new_order, 'customer_first', $meta["customer_first"][0]);
        update_post_meta($new_order, 'customer_last', $meta["customer_last"][0]);
        update_post_meta($new_order, 'customer_address', $meta["customer_address"][0]);
        update_post_meta($new_order, 'customer_phone', $meta["customer_phone"][0]);
        update_post_meta($new_order, 'product_id', $product_id);
        update_post_meta($new_order, 'product_name', get_the_title($product_id));
        update_post_meta($new_order, 'product_type', $product->get_type());
        update_post_meta($new_order, 'product_subtotal', $product->get_price());
        update_post_meta($new_order, 'product_currency', North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency()));
        update_post_meta($new_order, 'product_price_html', $product->get_price_html());
        update_post_meta($new_order, 'total', $product->get_price());
        update_post_meta($new_order, 'customer_email', $meta["customer_email"][0]);
        update_post_meta($new_order, 'card_brand', $meta["card_brand"][0]);
        update_post_meta($new_order, 'card_last_four', $meta["card_last_four"][0]);
        update_post_meta($new_order, 'product_trans_id', $payment_result['transaction_id']);

        if ($payment_result['success'] !== true) {
            echo "Product not added: " . esc_html( $payment_result['error']->getError()->message );
            update_post_meta($new_order, 'product_payment_status', "false");
        }else {
            update_post_meta($new_order, 'product_payment_status', "true");
        }

        do_action('north_after_payment_upsell');

        wp_die();
    }

    function paypal_order_create() {
        parse_str(isset($_POST['form']) ? $_POST['form'] : '', $form_data); //phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized, WordPress.Security.NonceVerification.Missing

        echo esc_html( $form_data['checkout_price'] );

        $order              = new North_Checkout_Order();
        $product_id         = $form_data['checkout_product'];
        $product            = new North_Checkout_Product($product_id);
        $bump_id            = $form_data['checkout_bump'];
        $flex_info          = array();
        $customer_address   = array();
        $variations         = array();
        $order_data         = array();
        $total              = 0;
        $quantity           = 1;
        $current_module = $form_data['product_module'];
        $product_payment_status = (isset($_POST['success']) && $_POST['success']) ? $_POST['success'] : 'false';
        $coupon                 = $form_data['checkout_discount_input'];

        $coupon = get_page_by_title($coupon, OBJECT, 'north_coupon');
        $coupon_price = 0;
        $coupon_type = null;

        if(!empty($coupon)) {
            $coupon_id = $coupon->ID;
            $coupon_status = $coupon->post_status;
            $coupon_instance = new North_Checkout_Coupon($coupon_id);
            $coupon_limit = intval(get_post_meta($coupon_id, 'coupon_limit', true));

            if($coupon_status == 'active' && (empty($coupon_limit) || intval($coupon_limit) > 0)) {

                $coupon_type = get_post_meta($coupon_id, 'coupon_type', true);
                $coupon_price = floatval(get_post_meta($coupon_id, 'coupon_price', true));
            }
        }

        if(isset($form_data['quantity_' . $current_module])) {
            $order_data['product_quantity'] = $form_data['quantity_' . $current_module];
            $quantity = intval($order_data['product_quantity']);
        }
        else {
            $order_data['product_quantity'] = $quantity;
        }

        $order_data['product_price_html'] = $product->get_price_html();
        $order_data['product_currency'] = North_Checkout_Admin::convert_code(North_Checkout_Admin::get_current_currency());
        $order_data['product_type'] = $product->get_type();
        $order_data['card_last_four'] = $form_data['card_last_four'];
        $order_data['card_brand'] = $form_data['card_brand'];
        $order_data['product_id'] = $product_id;
        $order_data['product_payment_status'] = $product_payment_status;
        $order_data['coupon'] = $coupon;

        // Write customer info
        foreach($form_data as $key => $value) {

            if (!empty($value)) {
                if( strpos($key, 'checkout_text_') !== false ) {

                    $flex_info['checkout' . substr($key, 13)] = $value;

                }elseif( strpos($key, 'checkout_radio_') !== false ) {

                    $flex_info['checkout' . substr($key, 14)] = $value;

                }elseif( strpos($key, 'checkout_checkbox_') !== false ) {

                    $flex_info['checkout' . substr($key, 17)] = $value;

                }elseif( strpos($key, 'checkout_select_') !== false ) {

                    $flex_info['checkout' . substr($key, 15)] = $value;

                }elseif( strpos($key, 'checkout_email_') !== false ) {

                    $order_data['checkout_email'] = $value;

                }elseif( strpos($key, 'checkout_first_') !== false ) {

                    $order_data['checkout_first'] = $value;

                }elseif( strpos($key, 'checkout_last_') !== false ) {

                    $order_data['checkout_last'] = $value;

                }elseif( strpos($key, 'checkout_phone_') !== false ) {

                    $order_data['checkout_phone'] = $value;

                }elseif( strpos($key, 'variation_') !== false  && $product->is_variable() ) {

                    if ($current_module === substr($key,strrpos($key, '_')+1)) {
                        $variable_key = substr($key, 10);
                        $variable_key = substr($variable_key, 0, strrpos($variable_key, "_"));
                        $variations[$variable_key] = $value;
                    }

                }
            }
        }

        if($product->get_type() == "Subscription") {
            if ($start_product_price = $product->get_start_price()) {
                $order_data['product_price'] = $start_product_price;
                $order_data['product_subtotal'] = $start_product_price;
                $order_data['total'] = $start_product_price;
            }else {
                $order_data['product_price'] = $product->get_price();
                $order_data['product_subtotal'] = $product->get_price();
                $order_data['total'] = $product->get_price();
            }

            if($product->get_trial_count()) {
                $order_data['product_subtotal'] = "0.00";
                $order_data['total'] = "0.00";
            }
        }elseif($product->get_type() == "Payment Plan") {
            if ($start_product_price = $product->get_start_price()) {
                $order_data['product_price'] = round(((floatval($product->get_price())) - floatval($start_product_price)) / intval($product->get_payment_count()), 2);
                $order_data['product_subtotal'] = round(((floatval($product->get_price())) - floatval($start_product_price)) / intval($product->get_payment_count()), 2);
                $order_data['total'] = round(((floatval($product->get_price())) - floatval($start_product_price)) / intval($product->get_payment_count()), 2);
            }else {
                $order_data['product_price'] = round(floatval($product->get_price()) / intval($product->get_payment_count()), 2);
                $order_data['product_subtotal'] = round(floatval($product->get_price()) / intval($product->get_payment_count()), 2);
                $order_data['total'] = round(floatval($product->get_price()) / intval($product->get_payment_count()), 2);
            }
        }elseif ($product->is_variable()) {
            $order_data['variations'] = $variations;
            $order_data['product_price'] = $product->get_variation_price($product_id, $variations);
            $order_data['product_subtotal'] = floatval($order_data['product_price']) * $quantity;
            $order_data['total'] = $order_data['product_subtotal'];

            if($coupon_type != null) {
                if($coupon_type == 'fixed') {
                    $order_data['total'] = $order_data['product_subtotal'] - $coupon_price;
                }else {
                    $percent = $order_data['product_subtotal'] / 100 * $coupon_price;
                    $order_data['total'] = $order_data['product_subtotal'] - $percent;
                }

                $order_data['total'] =  $order_data['total'] < 0 ? 0 :  $order_data['total'];
            }
        }else {
            $order_data['product_price'] = $product->get_price();
            $order_data['product_subtotal'] = floatval($order_data['product_price']) * $quantity;
            $order_data['total'] = $order_data['product_subtotal'];

            if($coupon_type != null) {
                if($coupon_type == 'fixed') {
                    $order_data['total'] = $order_data['product_subtotal'] - $coupon_price;
                }else {
                    $percent = $order_data['product_subtotal'] / 100 * $coupon_price;
                    $order_data['total'] = $order_data['product_subtotal'] - $percent;
                }
                $order_data['total'] =  $order_data['total'] < 0 ? 0 :  $order_data['total'];

            }
        }

        if(isset($form_data['checkout_address'])) {
            $customer_address = array(
                'full' => $form_data['checkout_address'] ? $form_data['checkout_address'] : 'Address not found',
                'line_2' => $form_data['checkout_line_2'] ? $form_data['checkout_line_2'] : '',
                'country' => $form_data['checkout_country'] ? $form_data['checkout_country'] : '',
                'city' => $form_data['checkout_city'] ? $form_data['checkout_city'] : '',
                'state' => $form_data['checkout_state'] ? $form_data['checkout_state'] : '',
                'zip' => $form_data['checkout_zip'] ?  $form_data['checkout_zip'] : '',
            );
        }

        $order_data['payment_method'] = 'PayPal';

        if($product_payment_status !== 'true') {
            $order_status = 'failed';
        }else {
            $order_status = 'complete';
        }



        if($order_data['total'] <= 0)
        {
            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status, 'Free');
        }
        else
        {
            $order_id = $order->create_order($order_data, $customer_address, $flex_info, $order_status);
        }

        if( !is_wp_error($order_id) ) {
            setcookie('order_id', $order_id, 0, "/");
        }

        wp_die(json_encode(array("success" => "true")));
    }

    function north_apply_coupon() {

        $coupon = $_POST['coupon'];
        $coupon = get_page_by_title($coupon, OBJECT, 'north_coupon');

        if(empty($coupon)) {
            wp_send_json_error('Сoupon is not valid');
        }

        $coupon_id = $coupon->ID;
        $coupon_status = $coupon->post_status;
        $coupon_instance = new North_Checkout_Coupon($coupon_id);
        $coupon_limit = get_post_meta($coupon_id, 'coupon_limit', true);

        if(!empty($coupon_limit) && intval($coupon_limit) < 1) {
            wp_send_json_error('Сoupon is not valid');
        }

        if($coupon_status != 'active') {
            wp_send_json_error('Сoupon is not valid');
        }

        $coupon_type = get_post_meta($coupon_id, 'coupon_type', true);
        $coupon_price = floatval(get_post_meta($coupon_id, 'coupon_price', true));

        wp_send_json_success( array('type' => $coupon_type, 'price' => $coupon_price, 'html' => '- ' . $coupon_instance->get_price_html()) );

    }
}